#!/usr/bin/python3
# Copyright (C) 2018 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Utility functions for dealing with copyright files."""

__all__ = [
    'NotMachineReadableError',
    'MachineReadableFormatError',
    'update_copyright',
    ]

from debian.copyright import (
    Copyright,
    MachineReadableFormatError,
    NotMachineReadableError,
    )

from .reformatting import Updater


class CopyrightUpdater(Updater):

    def __init__(self, path='debian/copyright'):
        super(CopyrightUpdater, self).__init__(path)

    def _parse(self, content):
        return Copyright(content, strict=False)

    def _format(self, parsed):
        return parsed.dump()

    @property
    def copyright(self):
        return self._parsed


def update_copyright(update_cb, path='debian/copyright'):
    """Update a machine-readable copyright file.

    Args:
      update_cb: Callback that will be called with the
        copyright file
      path: Optional path to the copyright file
    Returns:
      bool indicating whether any changes were made
    """
    with CopyrightUpdater(path=path) as updater:
        update_cb(updater.copyright)
    return updater.changed


def upstream_fields_in_copyright(path='debian/copyright'):
    ret = {}
    try:
        with open(path, 'r') as f:
            c = Copyright(f)
    except (FileNotFoundError, NotMachineReadableError,
            MachineReadableFormatError):
        return {}
    else:
        if c.header.upstream_contact:
            ret['Contact'] = c.header.upstream_contact
        if c.header.upstream_name:
            ret['Name'] = c.header.upstream_name
    return ret



#!/usr/bin/python3

from lintian_brush.control import update_control

default_priority = None
changed = []


def read_source_package(control):
    global default_priority
    default_priority = control.get('Priority')


def drop_library_priority(control):
    if control.get("Section") != 'libs':
        return
    priority = control.get('Priority', default_priority)
    if priority in ("required", "important", "standard"):
        control['Priority'] = 'optional'
        changed.append(control['Package'])


update_control(
    source_package_cb=read_source_package,
    binary_package_cb=drop_library_priority)

print('Set priority for library package%s %s to optional.' % (
      's' if len(changed) > 1 else '', ', '.join(changed)))
print('Fixed-Lintian-Tags: excessive-priority-for-library-package')



#!/usr/bin/python
# Copyright (C) 2019 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Tests for lintian_brush.yaml."""

from breezy.tests import (
    TestCaseInTempDir,
    TestCase,
    )

from ruamel.yaml.compat import ordereddict

from lintian_brush.yaml import (
    YamlUpdater,
    update_ordered_dict,
    )


class YamlUpdaterTests(TestCaseInTempDir):

    def test_new(self):
        with YamlUpdater('newfile.yaml') as code:
            code['Somekey'] = 'Somevalue'
        self.assertFileEqual("Somekey: Somevalue\n", "newfile.yaml")

    def test_update(self):
        with open('newfile.yaml', 'w') as f:
            f.write("""\
Origkey: origvalue
Somekey: origvalue
""")
        with YamlUpdater('newfile.yaml') as code:
            code['Somekey'] = 'Somevalue'
        self.assertFileEqual("""\
Origkey: origvalue
Somekey: Somevalue
""", "newfile.yaml")

    def test_delete(self):
        with open('newfile.yaml', 'w') as f:
            f.write("""\
Origkey: origvalue
Somekey: origvalue
""")
        with YamlUpdater('newfile.yaml') as code:
            del code['Origkey']
            del code['Somekey']
        self.assertPathDoesNotExist('newfile.yaml')

    def test_no_change(self):
        with open('newfile.yaml', 'w') as f:
            f.write("""\
Origkey: origvalue
Somekey: origvalue
""")
        with YamlUpdater('newfile.yaml'):
            pass
        self.assertFileEqual("""\
Origkey: origvalue
Somekey: origvalue
""", "newfile.yaml")

    def test_preserve_header(self):
        with open('newfile.yaml', 'w') as f:
            f.write("""\
---
Origkey: origvalue
Somekey: origvalue
""")
        with YamlUpdater('newfile.yaml') as code:
            code['Newkey'] = 'newvalue'
        self.assertFileEqual("""\
---
Origkey: origvalue
Somekey: origvalue
Newkey: newvalue
""", "newfile.yaml")


class UpdateOrderedDict(TestCase):

    def setUp(self):
        super(UpdateOrderedDict, self).setUp()
        self._od = ordereddict()

    def test_empty(self):
        update_ordered_dict(self._od, [('Contact', 'Foo'), ('Blah', 'blah')])
        self.assertEqual(ordereddict([
            ('Blah', 'blah'),
            ('Contact', 'Foo')]), self._od)

    def test_modify(self):
        self._od['Contact'] = 'Bar'
        self._od['ZZ'] = 'z'
        update_ordered_dict(
            self._od, [('Contact', 'Foo'), ('Blah', 'blah')])
        self.assertEqual(ordereddict([
            ('Blah', 'blah'),
            ('Contact', 'Foo'),
            ('ZZ', 'z'),
            ]), self._od)

    def test_insert_before(self):
        self._od['Contact'] = 'Bar'
        self._od['Bar'] = 'Bar'
        update_ordered_dict(
            self._od, [('Daar', 'blah')])
        self.assertEqual(ordereddict([
            ('Contact', 'Bar'),
            ('Bar', 'Bar'),
            ('Daar', 'blah'),
            ]), self._od)

    def test_csb(self):
        from ..upstream_metadata import upstream_metadata_sort_key
        self._od['Registry'] = [
                ordereddict([('Name', 'OMICtools'), ('Entry', 'OMICS_09827')]),
                ordereddict([('Name', 'bio.tools'), ('Entry', 'NA')])]
        self._od['Repository'] = 'https://github.com/csb-toolbox/CSB'
        update_ordered_dict(
            self._od,
            [('Bug-Database', 'https://github.com/csb-toolbox/CSB/issues'),
             ('Bug-Submit', 'https://github.com/csb-toolbox/CSB/issues/new'),
             ('Repository', 'https://github.com/csb-toolbox/CSB/issues.git')],
            key=upstream_metadata_sort_key)
        self.assertEqual(
            ordereddict([
                 ('Bug-Database', 'https://github.com/csb-toolbox/CSB/issues'),
                 ('Bug-Submit',
                  'https://github.com/csb-toolbox/CSB/issues/new'),
                 ('Registry', [
                     ordereddict([('Name', 'OMICtools'),
                                  ('Entry', 'OMICS_09827')]),
                     ordereddict([('Name', 'bio.tools'), ('Entry', 'NA')])
                  ]),
                 ('Repository',
                  'https://github.com/csb-toolbox/CSB/issues.git')]), self._od)



#!/usr/bin/python3

from lintian_brush.control import get_debhelper_compat_version
from lintian_brush.rules import (
    dh_invoke_drop_argument,
    dh_invoke_drop_with,
    update_rules,
    )

removed_args = []
unnecessary_args = []
unnecessary_with = []


compat_version = get_debhelper_compat_version()
if compat_version and compat_version >= 10:
    unnecessary_args.append(b'--parallel')
    unnecessary_with.append(b'systemd')


def drop_unnecessary_args(line, target):
    if not line.startswith(b'dh ') and not line.startswith(b'dh_'):
        return line
    for arg in unnecessary_args:
        newline = dh_invoke_drop_argument(line, arg)
        if newline != line:
            removed_args.append(arg)
            line = newline
    for arg in unnecessary_with:
        newline = dh_invoke_drop_with(line, arg)
        if newline != line:
            removed_args.append(b'--with=%s' % arg)
            line = newline
    return line


update_rules(drop_unnecessary_args)

print('Drop unnecessary dh arguments: %s' %
      ', '.join([arg.decode() for arg in removed_args]))
print('Fixed-Lintian-Tags: debian-rules-uses-unnecessary-dh-argument')



#!/usr/bin/python3

from lintian_brush.control import (
    update_control,
    add_dependency,
    drop_dependency,
    get_relation,
    iter_relations,
    )

added = []
removed = []
go_package = False
default_architecture = None


def check_go_package(control):
    global go_package, default_architecture
    if any(iter_relations(control.get('Build-Depends', ''), 'golang-go')):
        go_package = True
    if any(iter_relations(control.get('Build-Depends', ''), 'golang-any')):
        go_package = True
    default_architecture = control.get('Architecture')


def add_built_using(control):
    if control.get('Architecture', default_architecture) == 'all':
        if 'Built-Using' in control:
            control['Built-Using'] = drop_dependency(
                control['Built-Using'], '${misc:Built-Using}')
            if not control['Built-Using']:
                del control['Built-Using']
            removed.append(control['Package'])
    else:
        if go_package:
            built_using = control.get('Built-Using', '')
            try:
                get_relation(built_using, "${misc:Built-Using}")
            except KeyError:
                control["Built-Using"] = add_dependency(
                    built_using, "${misc:Built-Using}")
                added.append(control['Package'])


update_control(
    binary_package_cb=add_built_using, source_package_cb=check_go_package)

if added:
    print('Add missing ${misc:Built-Using} to Built-Using on %s.' %
          ', '.join(added))
if removed:
    print('Remove unnecessary ${misc:Built-Using} for %s' %
          ', '.join(removed))
print('Fixed-Lintian-Tags: '
      'missing-built-using-field-for-golang-package')



#!/usr/bin/python3

import os

from lintian_brush import certainty_to_confidence, certainty_sufficient
from lintian_brush.copyright import update_copyright, NotMachineReadableError
from lintian_brush.upstream_metadata import (
    guess_upstream_metadata_items,
    UpstreamDatum,
    )


minimum_certainty = os.environ.get('MINIMUM_CERTAINTY')
fields = []
achieved_certainty = []


def add_upstream_metadata(copyright):
    if copyright.header.upstream_name and copyright.header.upstream_contact:
        return
    import ruamel.yaml
    try:
        with open('debian/upstream/metadata', 'r') as f:
            inp = f.read()
    except FileNotFoundError:
        upstream_metadata = {}
    else:
        code = ruamel.yaml.safe_load(inp)
        upstream_metadata = {
            k: UpstreamDatum(k, v, 'certain') for (k, v) in code.items()}
    for datum in guess_upstream_metadata_items(
            '.', trust_package=(os.environ.get('TRUST_PACKAGE') == 'true')):
        if not certainty_sufficient(datum.certainty, minimum_certainty):
            continue
        if (datum.field not in upstream_metadata or
                upstream_metadata[datum.field].certainty != 'certain'):
            upstream_metadata[datum.field] = datum

    if not copyright.header.upstream_name:
        try:
            datum = upstream_metadata['Name']
        except KeyError:
            pass
        else:
            if datum.value:
                copyright.header.upstream_name = datum.value
                fields.append('Upstream-Name')
                achieved_certainty.append(datum.certainty)

    if not copyright.header.upstream_contact:
        try:
            datum = upstream_metadata['Contact']
        except KeyError:
            pass
        else:
            if datum.value:
                value = datum.value
                if isinstance(value, str):
                    value = [value]
                copyright.header.upstream_contact = value
                fields.append('Upstream-Contact')
                achieved_certainty.append(datum.certainty)


try:
    update_copyright(add_upstream_metadata)
except (FileNotFoundError, NotMachineReadableError):
    pass

if len(fields) == 1:
    print('Set field %s in debian/copyright.' % ', '.join(fields))
else:
    print('Set fields %s in debian/copyright.' % ', '.join(fields))
if achieved_certainty:
    print('Certainty: %s' %
          min(achieved_certainty, key=certainty_to_confidence))



#!/usr/bin/python3

from email.utils import parseaddr
from lintian_brush.control import ControlUpdater


with ControlUpdater() as updater:
    if ("Maintainer" in updater.source and
            parseaddr(updater.source["Maintainer"])[1] ==
            'packages@qa.debian.org'):
        try:
            del updater.source["Uploaders"]
        except KeyError:
            pass


print("Remove uploaders from orphaned package.")
print("Fixed-Lintian-Tags: orphaned-package-should-not-have-uploaders")



#!/usr/bin/python
# Copyright (C) 2018 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Tests for lintian_brush.systemd."""

from breezy.tests import (
    TestCaseWithTransport,
    )

from io import StringIO

from lintian_brush.systemd import (
    MissingSectionHeaderError,
    UnitFile,
    systemd_service_files,
    update_service_file,
    update_service,
    )


class UnitFileParser(TestCaseWithTransport):

    def test_multiple(self):
        f = StringIO("""\
[Service]
Before=a.service
Before=b.service
""")
        uf = UnitFile(f)
        self.assertEqual(['Service'], list(uf))
        self.assertEqual(
            ['a.service', 'b.service'],
            list(uf['Service']['Before']))

    def test_multiple_default(self):
        f = StringIO("""\
[Service]
""")
        uf = UnitFile(f)
        self.assertEqual([], list(uf['Service']['Before']))

    def test_multiple_add(self):
        f = StringIO("""\
[Service]
Before=a.service
Before=b.service
""")
        uf = UnitFile(f)
        uf['Service']['Before'] = ['a.service', 'b.service', 'c.service']
        self.assertEqual(['Service'], list(uf))
        self.assertEqual(
            ['a.service', 'b.service', 'c.service'],
            list(uf['Service']['Before']))
        self.assertEqual(str(uf), """\
[Service]
Before=a.service
Before=b.service
Before=c.service
""")

    def test_multiple_set(self):
        f = StringIO("""\
[Service]
Before=a.service
Before=b.service
""")
        uf = UnitFile(f)
        uf['Service']['Before'][0] = 'a.service'
        uf['Service']['Before'][1] = 'c.service'
        self.assertRaises(IndexError, uf['Service']['Before'].__getitem__, 3)
        self.assertEqual(
            ['a.service', 'c.service'],
            list(uf['Service']['Before']))
        self.assertEqual(str(uf), """\
[Service]
Before=a.service
Before=c.service
""")

    def test_multiple_remove(self):
        f = StringIO("""\
[Service]
Before=a.service
Before=b.service
""")
        uf = UnitFile(f)
        uf['Service']['Before'] = ['a.service']
        self.assertEqual(['Service'], list(uf))
        self.assertEqual(str(uf), """\
[Service]
Before=a.service
""")
        self.assertEqual(['a.service'], list(uf['Service']['Before']))

    def test_multiple_reset(self):
        f = StringIO("""\
[Service]
Before=a.service
Before=
Before=b.service
""")
        uf = UnitFile(f)
        self.assertEqual(['b.service'], list(uf['Service']['Before']))
        self.assertEqual(str(uf), """\
[Service]
Before=a.service
Before=
Before=b.service
""")

    def test_setting_before_section(self):
        f = StringIO("""\
Before=a.service

[Service]
Before=b.service
""")
        self.assertRaises(MissingSectionHeaderError, UnitFile, f)


class UpdateServiceFilesTests(TestCaseWithTransport):

    def setUp(self):
        super(UpdateServiceFilesTests, self).setUp()
        self.build_tree_contents([
            ('debian/', ),
            ('debian/other', ''),
            ('debian/apcupsd.service', """\
[Unit]
# network-online is really needed, otherwise there are problems with snmp
# -> 865620
After=network-online.target
Description=UPS power management daemon
Documentation=man:apcupsd(8)

""")])

    def test_systemd_service_files(self):
        self.assertEqual(
            ['debian/apcupsd.service'],
            list(systemd_service_files('debian')))

    def test_update_service_file(self):
        def cb(section, name, value):
            if section == "Unit" and name == "Description":
                return "Not really a UPS pmd"
            return value
        update_service_file('debian/apcupsd.service', cb)
        self.assertFileEqual("""\
[Unit]
# network-online is really needed, otherwise there are problems with snmp
# -> 865620
After=network-online.target
Description=Not really a UPS pmd
Documentation=man:apcupsd(8)

""", 'debian/apcupsd.service')

    def test_update_service(self):
        def cb(section, name, value):
            if section == "Unit" and name == "Description":
                return "Not really a UPS pmd"
            return value
        update_service(cb)
        self.assertFileEqual("""\
[Unit]
# network-online is really needed, otherwise there are problems with snmp
# -> 865620
After=network-online.target
Description=Not really a UPS pmd
Documentation=man:apcupsd(8)

""", 'debian/apcupsd.service')



#!/usr/bin/python3

from lintian_brush.control import ControlUpdater, delete_from_list


with ControlUpdater() as updater:
    if 'Uploaders' in updater.source:
        uploaders = updater.source['Uploaders'].split(',')
        maintainer = updater.source['Maintainer']
        if maintainer in [uploader.strip() for uploader in uploaders]:
            updater.source['Uploaders'] = delete_from_list(
                updater.source['Uploaders'], maintainer)


print("Remove maintainer from uploaders.")
print("Fixed-Lintian-Tags: maintainer-also-in-uploaders")



#!/usr/bin/python3

from lintian_brush.control import ensure_some_version, ControlUpdater
import os

needs_lsb_base = set()
other_service_present = set()

for n in os.listdir('debian'):
    if n.endswith('.init'):
        with open(os.path.join('debian', n), 'r') as f:
            for l in f:
                if l.startswith('. /lib/lsb/'):
                    needs_lsb_base.add(n.split('.')[0])
                    break
    # TODO(jelmer): This doesn't find all service files; they may not live in
    # the debian/ directory.
    if n.endswith('.service'):
        other_service_present.add(n.split('.')[0].rstrip('@'))

# TODO(jelmer): Also check that there are no implicit dependencies on lsb-base.
needs_lsb_base -= other_service_present

if needs_lsb_base:
    with ControlUpdater() as updater:
        for binary in updater.binaries:
            if binary['Package'] not in needs_lsb_base:
                continue
            binary['Depends'] = ensure_some_version(
                binary['Depends'], 'lsb-base')

print('Add missing dependency on lsb-base.')
print('Fixed-Lintian-Tags: init.d-script-needs-depends-on-lsb-base')
print('Certainty: possible')



#!/usr/bin/python3

from lintian_brush.control import ControlUpdater


with ControlUpdater() as updater:
    for para in updater.paragraphs:
        if para.get("Priority") == "extra":
            para["Priority"] = "optional"


print("Change priority extra to priority optional.")
print("Fixed-Lintian-Tags: priority-extra-is-replaced-by-priority-optional")



#!/usr/bin/python3

import asyncio
from functools import partial
from lintian_brush.changelog import ChangelogUpdater
from lintian_brush.fixer import net_access_allowed
import re
import socket
from warnings import warn

certainty = 'certain'
debbugs = None
tags = set()


async def valid_bug(package, bug):
    if not net_access_allowed():
        return None
    global debbugs
    if debbugs is None:
        from lintian_brush.debbugs import DebBugs
        debbugs = DebBugs()
        try:
            await debbugs.connect()
        except ImportError:
            # No asynpcg?
            return None
        except socket.gaierror as e:
            warn('Unable to connect to debbugs: %s' % e)
            return None
    return await debbugs.check_bug(package, bug)


def check_bug(package, m):
    global certainty
    bug = int(m.group('bug'))
    loop = asyncio.get_event_loop()
    valid = loop.run_until_complete(valid_bug(package, bug))
    if valid is None:
        # Let's assume valid, but downgrade certainty
        valid = True
        # Check number of digits; upstream projects don't often hit the 5-digit
        # bug numbers that Debian has.
        if len(m.group('bug')) >= 5:
            certainty = 'likely'
        else:
            certainty = 'possible'
    if valid:
        tags.add("possible-missing-colon-in-closes")
        return '%s: #%d' % (m.group('closes'), bug)
    else:
        return m.group(0)


def fix_close_typo(m):
    tags.add('misspelled-closes-bug')
    return '%ss: #%s' % (m.group('close'), m.group('bug'))


with ChangelogUpdater() as updater:
    for block in updater.changelog:
        for i, change in enumerate(block._changes):
            change = re.sub(
                r'(?<!partially )(?P<closes>closes) '
                r'#(?P<bug>[0-9]+)',
                partial(check_bug, block.package), change,
                flags=re.IGNORECASE)
            change = re.sub(
                '(?P<close>close): #(?P<bug>[0-9]+)',
                fix_close_typo, change, flags=re.IGNORECASE)
            block._changes[i] = change


if tags == set(['possible-missing-colon-in-closes']):
    print("Add missing colon in closes line.")
elif tags == set(['misspelled-closes-bug']):
    print("Fix misspelling of Close => Closes.")
else:
    print("Fix formatting of bug closes.")
print("Fixed-Lintian-Tags: %s" % ", ".join(sorted(tags)))
print("Certainty: %s" % certainty)



#!/usr/bin/python3
# Copyright (C) 2019 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Utility functions for dealing with YAML files."""

import copy
from io import StringIO
import os
from ruamel.yaml import YAML


class YamlUpdater(object):

    def __init__(self, path, remove_empty=True):
        self.yaml = YAML()
        self.path = path
        self._dirpath = os.path.dirname(path)
        self.remove_empty = remove_empty
        self._directives = []

    def __enter__(self):
        try:
            with open(self.path, 'r') as f:
                inp = list(f)
        except FileNotFoundError:
            self._code = {}
        else:
            if '---\n' in inp:
                for i, line in enumerate(inp):
                    if line == '---\n':
                        self._directives = inp[:i+1]
                        break
            self._code = self.yaml.load(''.join(inp))
        self._orig = copy.deepcopy(self._code)
        return self._code

    def _only_simple_changes(self):
        def is_one_line(k, v):
            f = StringIO()
            self.yaml.dump({k: v}, f)
            return len(f.getvalue().splitlines()) == 1
        # Check if there are only "simple" changes, i.e.
        # changes that update string fields
        for k, v in self._code.items():
            if v == self._orig.get(k) or not is_one_line(k, v):
                continue
        for k, v in self._orig.items():
            if k not in self._code and not is_one_line(k, v):
                return False
        return True

    def _update_lines(self, lines, f):
        for line in self._directives:
            f.write(line)
        os = list(self._orig.keys())
        cs = list(self._code.keys())
        o = 0
        for line in lines[len(self._directives):]:
            try:
                key = os[o]
            except IndexError:
                key = None
            if key and line.startswith(key + ':'):
                while cs and cs[0] not in os:
                    self.yaml.dump({cs[0]: self._code[cs[0]]}, f)
                    cs.pop(0)
                if key not in self._code:
                    pass  # Line was removed
                elif self._code[key] == self._orig[key]:
                    # Line stayed the same
                    f.write(line)
                    cs.remove(key)
                else:
                    self.yaml.dump({key: self._code[key]}, f)
                    cs.remove(key)
                o += 1
            else:
                f.write(line)
        while cs:
            key = cs.pop(0)
            self.yaml.dump({key: self._code[key]}, f)

    def __exit__(self, exc_type, exc_val, exc_tb):
        if not exc_type:
            if not self._code and self.remove_empty:
                if os.path.exists(self.path):
                    os.unlink(self.path)
                    if self._dirpath and not os.listdir(self._dirpath):
                        os.rmdir(self._dirpath)
            else:
                if self._code != self._orig:
                    if not os.path.exists(self._dirpath) and self._dirpath:
                        os.mkdir(self._dirpath)
                    if self._only_simple_changes():
                        try:
                            with open(self.path, 'r') as f:
                                lines = list(f.readlines())
                        except FileNotFoundError:
                            lines = []
                        with open(self.path, 'w') as f:
                            self._update_lines(lines, f)
                    else:
                        with open(self.path, 'w') as f:
                            f.writelines(self._directives)
                            self.yaml.dump(self._code, f)
        return False


def update_ordered_dict(code, changed, key=None):
    if key is None:
        def key(x):
            return x

    to_insert = []
    for k, v in changed:
        if k in code:
            code[k] = v
        else:
            to_insert.append((k, v))

    to_insert.sort(key=key)
    i = 0
    for k, v in list(code.items()):
        while to_insert and key((k, v)) > key(to_insert[0]):
            code.insert(i, *to_insert.pop(0))
            i += 1
        i += 1

    while to_insert:
        k, v = to_insert.pop(0)
        code[k] = v



#!/usr/bin/python3
from lintian_brush.control import ControlUpdater


with ControlUpdater() as updater:
    for key in list(updater.source):
        if key.startswith('XS-Vcs-'):
            updater.source[key[3:]] = updater.source[key]
            del updater.source[key]


print("Remove unnecessary XS- prefix for Vcs- fields in debian/control.")
print("Fixed-Lintian-Tags: xs-vcs-field-in-debian-control")



#!/usr/bin/python3
# Copyright (C) 2018 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Utility functions for dealing with deb822 files."""

from debian.deb822 import Deb822
from io import BytesIO
from .reformatting import (
    Updater,
    )


def dump_paragraphs(paragraphs):
    """Dump a set of deb822 paragraphs to a file.

    Args:
      paragraphs: iterable over paragraphs
    Returns:
      formatted text (as bytes)
    """
    outf = BytesIO()
    first = True
    for paragraph in paragraphs:
        if paragraph:
            if not first:
                outf.write(b'\n')
            paragraph.dump(fd=outf, encoding='utf-8')
            first = False
    return outf.getvalue()


def reformat_deb822(contents):
    """Check whether it's possible to preserve a control file.

    Args:
      contents: Original contents
    Returns:
      New contents
    """
    return dump_paragraphs(
        Deb822.iter_paragraphs(BytesIO(contents), encoding='utf-8'))


class Deb822Updater(Updater):

    def __init__(self, path, allow_generated=False):
        super(Deb822Updater, self).__init__(
            path, allow_generated=allow_generated, mode='b')

    def apply_changes(self, changes):
        changes = dict(changes.items())
        for paragraph in self.paragraphs:
            for item in paragraph.items():
                for key, value in changes.pop(item, []):
                    if value is None:
                        del paragraph[key]
                    else:
                        paragraph[key] = value
        self.paragraphs.extend([Deb822(dict(p)) for p in changes.values()])

    def _parse(self, content):
        return list(Deb822.iter_paragraphs(content, encoding='utf-8'))

    @property
    def paragraphs(self):
        return self._parsed

    def _format(self, paragraphs):
        return dump_paragraphs(paragraphs)



#!/usr/bin/python3

import re
import sys

from lintian_brush.control import get_debhelper_compat_version
from lintian_brush.rules import update_rules

compat_version = get_debhelper_compat_version()
if compat_version is None:
    sys.exit(0)


def drop_explicit_dh_compat(line):
    if re.match(b'export DH_COMPAT[ \t]*=', line):
        return []
    return line


update_rules(global_line_cb=drop_explicit_dh_compat)
print('Avoid setting debhelper compat version in debian/rules '
      'and debian/compat.')
print('Fixed-Lintian-Tags: '
      'declares-possibly-conflicting-debhelper-compat-versions')



#!/usr/bin/python
# Copyright (C) 2018 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Tests for lintian_brush.copyright."""

from debian.copyright import (
    FilesParagraph,
    License,
    )

from breezy.tests import (
    TestCaseWithTransport,
    )

from lintian_brush.copyright import (
    NotMachineReadableError,
    update_copyright,
    )
from lintian_brush.reformatting import (
    FormattingUnpreservable,
    )


class UpdateControlTests(TestCaseWithTransport):

    def test_unpreservable(self):
        self.build_tree_contents([('debian/', ), ('debian/copyright', """\
Format: https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
Upstream-Name: lintian-brush
Upstream-Contact: Jelmer <jelmer@samba.org>


Files: *
License: GPL
Copyright: 2012...
""")])

        def dummy(cb):
            cb.header.upstream_name = 'llintian-brush'
        self.assertRaises(FormattingUnpreservable, update_copyright, dummy)

    def test_old_style(self):
        self.build_tree_contents([('debian/', ), ('debian/copyright', """\
This package was debianized in 1995 by Joe Example <joe@example.com>

It was downloaded from ftp://ftp.example.com/pub/blah.
""")])
        self.assertRaises(NotMachineReadableError, update_copyright, None)

    def test_modify(self):
        self.build_tree_contents([('debian/', ), ('debian/copyright', """\
Format: https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
Upstream-Name: lintian-brush
Upstream-Contact: Jelmer <jelmer@samba.org>

Files: *
License: GPL
Copyright: 2012...
""")])

        def add_stanza(copyright):
            copyright.add_files_paragraph(FilesParagraph.create(
                ['foo.c'], "2012 Joe Example",
                License("Apache")))
        self.assertTrue(update_copyright(add_stanza))
        self.assertFileEqual("""\
Format: https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
Upstream-Name: lintian-brush
Upstream-Contact: Jelmer <jelmer@samba.org>

Files: *
License: GPL
Copyright: 2012...

Files: foo.c
Copyright: 2012 Joe Example
License: Apache
""", 'debian/copyright')



#!/usr/bin/python3

import os

from debian.changelog import Version

from lintian_brush.control import (
    drop_dependency,
    ensure_exact_version,
    get_relation,
    read_debian_compat_file,
    ControlUpdater,
    )
from lintian_brush.rules import (
    check_cdbs,
    )


if os.path.exists('debian/compat'):
    # Package currently stores compat version in debian/compat..

    debhelper_compat_version = read_debian_compat_file('debian/compat')

    # debhelper >= 11 supports the magic debhelper-compat build-dependency.
    # Exclude cdbs, since it only knows to get the debhelper compat version
    # from debian/compat.

    if debhelper_compat_version >= 11 and not check_cdbs():
        # Upgrade to using debhelper-compat, drop debian/compat file.
        os.unlink('debian/compat')

        # Assume that the compat version is set in Build-Depends
        with ControlUpdater() as updater:
            # TODO(jelmer): Use iter_relations rather than get_relation,
            # since that allows for complex debhelper rules.
            try:
                position, debhelper_relation = get_relation(
                    updater.source.get("Build-Depends", ""), "debhelper")
            except KeyError:
                position = None
                debhelper_relation = []
            updater.source["Build-Depends"] = ensure_exact_version(
                updater.source.get("Build-Depends", ""), "debhelper-compat",
                "%d" % debhelper_compat_version, position=position)
            current_compat_version = Version("%d" % debhelper_compat_version)
            # If there are debhelper dependencies >> new debhelper compat
            # version, then keep them.
            for rel in debhelper_relation:
                if not rel.version:
                    continue
                if rel.version[0] in ('=', '>=') and Version(
                        rel.version[1]) > current_compat_version:
                    break
                if rel.version[0] == '>>' and Version(
                        rel.version[1]) >= current_compat_version:
                    break
            else:
                updater.source["Build-Depends"] = drop_dependency(
                    updater.source.get("Build-Depends", ""), "debhelper")
                if updater.source.get("Build-Depends") == "":
                    del updater.source["Build-Depends"]

print("Set debhelper-compat version in Build-Depends.")
print("Fixed-Lintian-Tags: uses-debhelper-compat-file")



#!/usr/bin/python3

from lintian_brush.control import ControlUpdater


with ControlUpdater() as updater:
    try:
        del updater.source["DM-Upload-Allowed"]
    except KeyError:
        pass


print("Remove malformed and unnecessary DM-Upload-Allowed field in "
      "debian/control.")
print("Fixed-Lintian-Tags: malformed-dm-upload-allowed")



#!/usr/bin/python3

import os

try:
    with open('debian/tests/control', 'r') as f:
        if f.read().strip() == '':
            os.unlink('debian/tests/control')
            if not os.listdir('debian/tests'):
                os.rmdir('debian/tests')
except FileNotFoundError:
    pass

print('Remove empty debian/tests/control.')
print('Fixed-Lintian-Tags: empty-debian-tests-control')



#!/usr/bin/python3
import sys

from lintian_brush.control import (
    drop_dependency,
    ensure_minimum_debhelper_version,
    update_control,
    )
from lintian_brush.debhelper import maximum_debhelper_compat_version
from lintian_brush.fixer import compat_release
from lintian_brush.rules import (
    dh_invoke_drop_with,
    update_rules,
    )


def bump_debhelper(control):
    control["Build-Depends"] = ensure_minimum_debhelper_version(
        control.get("Build-Depends", ""), "10~")
    control["Build-Depends"] = drop_dependency(
        control["Build-Depends"],
        "dh-autoreconf")


def drop_with_autoreconf(line, target):
    return dh_invoke_drop_with(line, b'autoreconf')


if maximum_debhelper_compat_version(compat_release()) < 10:
    sys.exit(0)

changed = update_rules(drop_with_autoreconf)
if not changed:
    sys.exit(2)


update_control(source_package_cb=bump_debhelper)

print("Drop unnecessary dependency on dh-autoreconf.")
print("Fixed-Lintian-Tags: useless-autoreconf-build-depends")



#!/usr/bin/python3

from lintian_brush.control import update_control

updated_packages = set()


def set_multiarch_foreign(binary):
    package = binary['Package']
    if (not package.startswith('fonts-') and
            not package.startswith('xfonts-')):
        return
    if binary.get('Architecture') not in ('all', None):
        return
    if 'Multi-Arch' in binary:
        return
    binary['Multi-Arch'] = 'foreign'
    updated_packages.add(package)


update_control(binary_package_cb=set_multiarch_foreign)

print('Set Multi-Arch: foreign on package%s %s.' % (
    's' if len(updated_packages) > 1 else '', ', '.join(updated_packages)))
print("Fixed-Lintian-Tags: font-packge-not-multi-arch-foreign")



#!/usr/bin/python3

from lintian_brush.upstream_metadata import ADDON_ONLY_FIELDS
from lintian_brush.yaml import YamlUpdater

obsolete_fields = set()
removed_fields = []

with YamlUpdater('debian/upstream/metadata') as code:

    # If the debian/copyright file is machine-readable, then we can drop the
    # Name/Contact information from the debian/upstream/metadata file.
    if 'Name' in code or 'Contact' in code:
        from lintian_brush.copyright import upstream_fields_in_copyright
        obsolete_fields.update(
            upstream_fields_in_copyright('debian/copyright'))

    for field in obsolete_fields:
        if field in code:
            del code[field]
            removed_fields.append(field)

    if removed_fields:
        if not (set(code.keys()) - set(ADDON_ONLY_FIELDS)):
            code.clear()


print('Remove obsolete field%s %s from debian/upstream/metadata '
      '(already present in machine-readable debian/copyright).' %
      ('s' if len(removed_fields) > 1 else '',
       ', '.join(sorted(removed_fields))))



#!/usr/bin/python3

import os
import subprocess
import shlex
import sys

gpg = shlex.split(os.environ.get('GPG', 'gpg'))


KEY_BLOCK_START = b'-----BEGIN PGP PUBLIC KEY BLOCK-----'
KEY_BLOCK_END = b'-----END PGP PUBLIC KEY BLOCK-----'


def gpg_import_export(options, stdin):
    argv = gpg + [
            '--armor', '--quiet', '--no-default-keyring',
            '--import-options', ','.join(['import-export'] + options)]
    try:
        p = subprocess.Popen(
            argv, stdout=subprocess.PIPE, stdin=subprocess.PIPE)
    except FileNotFoundError:
        # No gpg, no dice.
        sys.exit(2)
    (stdout, stderr) = p.communicate(stdin, timeout=5)
    if p.returncode != 0:
        raise Exception('gpg failed')
    return stdout


def minimize_key_block(key):
    minimal = gpg_import_export(
        ['import-minimal', 'import-clean', 'self-sigs-only'], key)
    full = gpg_import_export([], key)
    if minimal == full:
        return key
    else:
        return minimal


for p in [
        'debian/upstream/signing-key.asc',
        'debian/upstream/signing-key.pgp',
        'debian/upstream-signing-key.pgp']:
    if os.path.exists(p):
        out = []
        key = None
        with open(p, 'rb') as f:
            for line in f:
                if line.strip() == KEY_BLOCK_START:
                    key = [line]
                elif line.strip() == KEY_BLOCK_END:
                    key.append(line)
                    key = minimize_key_block(b''.join(key)).splitlines(True)
                    out.extend(key)
                    key = None
                elif key is not None:
                    key.append(line)
                else:
                    out.append(line)
            if key:
                raise Exception('Key block without end')
        with open(p, 'wb') as f:
            f.writelines(out)


print("Re-export upstream signing key without extra signatures.")
print("Fixed-Lintian-Tags: public-upstream-key-not-minimal")



#!/usr/bin/python3

from lintian_brush.watch import WatchUpdater
from urllib.parse import urlparse


with WatchUpdater() as updater:
    for w in getattr(updater.watch_file, 'entries', []):
        parsed_url = urlparse(w.url)
        if parsed_url.netloc != 'githubredir.debian.net':
            continue
        parts = parsed_url.path.strip('/').split('/')
        if parts[0] != 'github':
            # Hmm.
            continue
        w.url = 'https://github.com/%s/%s/tags' % (parts[1], parts[2])
        w.matching_pattern = '.*/' + w.matching_pattern.rsplit('/', 1)[-1]


print(
    'Remove use of githubredir - see '
    'https://lists.debian.org/debian-devel-announce/2014/10/msg00000.html '
    'for details.')
print('Fixed-Lintian-Tags: debian-watch-file-uses-deprecated-githubredir')



#!/usr/bin/python
# Copyright (C) 2019 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Tests for lintian_brush.multiarch_hints."""

from io import BytesIO

from breezy.tests import (
    TestCase,
    )

from lintian_brush.multiarch_hints import (
    parse_multiarch_hints,
    )


class ParseMultiArchHints(TestCase):

    def test_invalid_header(self):
        f = BytesIO(b"""\
format: blah
""")
        self.assertRaises(ValueError, parse_multiarch_hints, f)

    def test_some_entries(self):
        f = BytesIO(b"""\
format: multiarch-hints-1.0
hints:
- binary: coinor-libcoinmp-dev
  description: coinor-libcoinmp-dev conflicts on ...
  link: https://wiki.debian.org/MultiArch/Hints#file-conflict
  severity: high
  source: coinmp
  version: 1.8.3-2+b11
""")
        self.assertEqual(
            parse_multiarch_hints(f), [{
                'binary': 'coinor-libcoinmp-dev',
                'description': 'coinor-libcoinmp-dev conflicts on ...',
                'link': (
                    'https://wiki.debian.org/MultiArch/Hints#file-conflict'),
                'severity': 'high',
                'source': 'coinmp',
                'version': '1.8.3-2+b11'
              }],
            )



#!/usr/bin/python3
# Copyright (C) 2018 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Utility functions for dealing with lintian overrides files."""

import collections
import os


# https://lintian.debian.org/manual/section-2.4.html
# File format (as documented in policy 2.4.1):
# [[<package>][ <archlist>][ <type>]: ]<lintian-tag>[ [*]<lintian-info>[*]]


VALID_TYPES = ['udeb', 'source', 'binary']
Override = collections.namedtuple(
    'Override', ['package', 'archlist', 'type', 'tag', 'info'])
Override.__new__.__defaults__ = (None,) * len(Override._fields)


def overrides_paths():
    for path in ['debian/source/lintian-overrides']:
        if os.path.exists(path):
            yield path


def update_overrides(cb):
    """"Call update_overrides_file on all overrides files.

    Args:
      cb: Callback that modifies overrides; called with an Override object
    """
    for path in overrides_paths():
        update_overrides_file(cb, path=path)


def parse_override(line):
    """Parse an override line

    Args:
      line: Line to parse
    Returns:
      An Override object
    Raises:
      ValueError: when encountering invalid syntax
    """
    line = line.strip()
    archlist = None
    package = None
    type = None
    if ': ' in line:
        origin, issue = line.split(': ', 1)
        while origin:
            origin = origin.strip()
            if origin.startswith('['):
                archlist, origin = origin[1:].split(']', 1)
            else:
                try:
                    field, origin = origin.split(' ', 1)
                except ValueError:
                    field = origin
                    origin = ''
                if field in VALID_TYPES:
                    type = field
                else:
                    package = field
    else:
        issue = line
    try:
        tag, info = issue.split(None, 1)
    except ValueError:
        tag = issue
        info = None
    return Override(
        package=package, archlist=archlist, type=type, tag=tag, info=info)


def serialize_override(override):
    """Serialize an override.

    Args:
      override: An Override object
    Returns:
      serialized override, including newline
    """
    origin = []
    if override.package:
        origin.append(override.package)
    if override.archlist:
        origin.append('[' + override.archlist + ']')
    if override.type:
        origin.append(override.type)
    if origin:
        line = ' '.join(origin) + ': ' + override.tag
    else:
        line = override.tag
    if override.info:
        line += ' ' + override.info
    return line + '\n'


def update_overrides_file(cb, path='debian/source/lintian-overrides'):
    """Modify the overrides in a file.

    Args:
      cb: Callback that modifies overrides; called with an Override object
        Should return new override or None to delete override.
    Returns:
        Whether the file was modified
    """
    changed = False
    lines = []
    with open(path, 'r') as f:
        for line in f.readlines():
            if line.startswith('#') or not line.strip():
                lines.append(line)
            else:
                old = parse_override(line)
                new = cb(old)
                if old != new:
                    changed = True
                    if new is not None:
                        lines.append(serialize_override(new))
                else:
                    lines.append(line)

    if not changed:
        return False

    if lines:
        with open(path, 'w') as f:
            f.writelines(lines)
    else:
        os.unlink(path)

    return True


def _get_overrides(package=None):
    if package in ('source', None):
        paths = ['debian/source/lintian-overrides',
                 'debian/source.lintian-overrides']
    else:
        paths = ['debian/%s.lintian-overrides' % package]

    for path in paths:
        try:
            with open(path, 'r') as f:
                for line in f.readlines():
                    if line.startswith('#') or not line.strip():
                        pass
                    else:
                        yield parse_override(line)
        except FileNotFoundError:
            pass


def override_exists(tag, info=None, package=None):
    """Check if a particular override exists.

    Args:
      tag: Tag name
      info: Optional info
      package: Package (as type, name tuple)
    """
    for override in _get_overrides(package):
        if override.tag != tag:
            continue
        if override.info and info != override.info:
            continue
        return True
    return False


async def get_unused_overrides(packages):
    from .udd import connect_udd_mirror
    udd = await connect_udd_mirror()

    args = []
    extra = []
    for (type, name) in packages:
        extra.append('package = $%d AND package_type = $%d' % (
            len(args)+1, len(args)+2))
        args.extend([name, type])

    return list(await udd.fetch(
        """\
select package, package_type, package_version, information
from lintian where tag = 'unused-override' AND (%s)""" % " OR ".join(extra),
        *args))


def remove_unused():
    from debian.deb822 import Deb822
    packages = []
    with open('debian/control', 'r') as f:
        for para in Deb822.iter_paragraphs(f):
            if 'Source' in para:
                packages.append(('source', para['Source']))
            else:
                packages.append(('binary', para['Package']))
    global unused_overrides
    unused_overrides = None
    removed = []

    def is_unused(override, unused_overrides):
        for unused_override in unused_overrides:
            if override.package not in (None, unused_override[0]):
                continue
            if override.type not in (None, unused_override[1]):
                continue
            if override.info:
                expected_info = '%s %s' % (override.tag, override.info)
            else:
                expected_info = override.tag
            if expected_info != unused_override[3]:
                continue
            return True
        return False

    def drop_override(override):
        global unused_overrides
        if unused_overrides is None:
            import asyncio
            loop = asyncio.get_event_loop()
            unused_overrides = loop.run_until_complete(
                get_unused_overrides(packages))
        if is_unused(override, unused_overrides):
            removed.append(override)
            return None
        return override
    update_overrides(drop_override)
    return removed


if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--remove-unused', action='store_true',
        help='Remove unused overrides.')
    args = parser.parse_args()
    if args.remove_unused:
        removed = remove_unused()
        print('Removed %d unused overrides' % len(removed))
    else:
        parser.print_usage()



#!/usr/bin/python3

import os
import sys

try:
    with open('debian/source/options', 'r') as f:
        oldlines = list(f.readlines())
except FileNotFoundError:
    sys.exit(0)


def drop_prior_comments(lines):
    while lines and lines[-1].strip().startswith('#'):
        lines.pop(-1)


newlines = []
for line in oldlines:
    if line.lstrip().startswith('#'):
        newlines.append(line)
        continue
    try:
        (key, value) = line.split('=', 1)
    except ValueError:
        newlines.append(line)
    else:
        if key.strip() == 'compression':
            drop_prior_comments(newlines)
            print("Drop custom source compression.")
            continue
        if key.strip() == 'compression-level':
            drop_prior_comments(newlines)
            print("Drop custom source compression level.")
            continue
        newlines.append(line)

if newlines:
    with open('debian/source/options', 'w') as f:
        f.writelines(newlines)
else:
    os.unlink('debian/source/options')

print(
    "Fixed-Lintian-Tags: debian-source-options-has-custom-compression-settings"
    )



#!/usr/bin/python3

from lintian_brush.copyright import CopyrightUpdater, NotMachineReadableError
from lintian_brush.fixer import report_result
import sys
import re

used = set()
defined = set()
certainty = 'certain'


def extract_licenses(synopsis):
    ret = []
    for license in synopsis.split(" or "):
        m = re.fullmatch(r'(.*) with (.*) exception', license)
        if m:
            license = m.group(1)
        ret.append(license)
    return ret


try:
    with CopyrightUpdater() as updater:
        if updater.copyright.header.license:
            if updater.copyright.header.license.text:
                defined.add(updater.copyright.header.license.synopsis)
        for paragraph in updater.copyright.all_paragraphs():
            if not paragraph.license:
                continue
            if paragraph.license.text:
                defined.add(paragraph.license.synopsis)

        if updater.copyright.header.license:
            synopsis = updater.copyright.header.license.synopsis
            if synopsis:
                if synopsis in defined:
                    used.add(synopsis)
                used.update(extract_licenses(synopsis))

        for paragraph in updater.copyright.all_files_paragraphs():
            if not paragraph.license:
                continue
            if paragraph.files:
                synopsis = paragraph.license.synopsis
                if synopsis in defined:
                    used.add(synopsis)
                used.update(extract_licenses(synopsis))

        extra_defined = (defined - used)
        extra_used = (used - defined)

        if extra_used:
            sys.stderr.write('Undefined licenses in copyright: %r' %
                             extra_used)
            # Drop the certainty since it's possible the undefined licenses
            # are actually the referenced ones.
            certainty = 'possible'

        for name in extra_defined:
            for paragraph in updater.copyright.all_paragraphs():
                if not paragraph.license:
                    continue
                if paragraph.license.synopsis == name:
                    continue
                if paragraph.license.text and name in paragraph.license.text:
                    certainty = 'possible'
                if paragraph.comment and name in paragraph.comment:
                    certainty = 'possible'

        if extra_defined and not extra_used:
            for paragraph in list(updater.copyright._Copyright__paragraphs):
                if not paragraph.license:
                    continue
                if paragraph.license.synopsis in extra_defined:
                    updater.copyright._Copyright__paragraphs.remove(paragraph)
except (FileNotFoundError, NotMachineReadableError):
    pass
else:
    report_result(
        "Remove unused license definitions for %s." % ', '.join(extra_defined),
        fixed_lintian_tags=['unused-license-paragraph-in-dep5-copyright'],
        certainty=certainty)



#!/usr/bin/python3

from lintian_brush.control import ControlUpdater
from lintian_brush.fixer import report_result


with ControlUpdater() as updater:
    # Remove anything that involves python 2.6, 2.7, 3.3
    if "X-Python-Version" in updater.source:
        if updater.source["X-Python-Version"].strip().startswith(">= 2."):
            del updater.source["X-Python-Version"]
    if ("X-Python3-Version" in updater.source and
            updater.source["X-Python3-Version"].strip().startswith(">=")):
        vers = updater.source["X-Python3-Version"].split(">=")[1].strip()
        if vers in ("3.0", "3.1", "3.2", "3.3", "3.4"):
            del updater.source["X-Python3-Version"]


report_result(
    "Remove unnecessary X-Python{,3}-Version field in debian/control.",
    fixed_lintian_tags=['ancient-python-version-field'])



#!/usr/bin/python3

from lintian_brush.control import update_control


def drop_unnecessary_autopkgtest(source):
    if "Testsuite" not in source:
        return
    if source["Testsuite"] == "autopkgtest":
        del source["Testsuite"]


update_control(source_package_cb=drop_unnecessary_autopkgtest)

print("Remove unnecessary 'Testsuite: autopkgtest' header.")
print("Fixed-Lintian-Tags: unnecessary-testsuite-autopkgtest-field")



#!/usr/bin/python
# Copyright (C) 2018 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Tests for lintian_brush.control."""

from breezy.tests import (
    TestCase,
    TestCaseWithTransport,
    )

from lintian_brush.control import (
    add_dependency,
    drop_dependency,
    ensure_exact_version,
    ensure_minimum_version,
    ensure_some_version,
    get_relation,
    iter_relations,
    is_relation_implied,
    is_dep_implied,
    update_control,
    PkgRelation,
    format_relations,
    parse_relations,
    delete_from_list,
    ControlUpdater,
    )
from lintian_brush.reformatting import (
    GeneratedFile,
    FormattingUnpreservable,
    )


class UpdateControlTests(TestCaseWithTransport):

    def test_do_not_edit(self):
        self.build_tree_contents([('debian/', ), ('debian/control', """\
# DO NOT EDIT
# This file was generated by blah

Source: blah
Testsuite: autopkgtest

""")])

        def source_cb(c):
            c['Source'] = 'blah1'
        self.assertRaises(
            GeneratedFile, update_control, source_package_cb=source_cb)

    def test_do_not_edit_no_change(self):
        self.build_tree_contents([('debian/', ), ('debian/control', """\
# DO NOT EDIT
# This file was generated by blah

Source: blah
Testsuite: autopkgtest

""")])
        update_control()

    def test_unpreservable(self):
        self.build_tree_contents([('debian/', ), ('debian/control', """\
Source: blah
# A comment
Testsuite: autopkgtest

""")])

        def update_source(control):
            control["NewField"] = "New Field"
        self.assertRaises(
            FormattingUnpreservable, update_control,
            source_package_cb=update_source)

    def test_modify_source(self):
        self.build_tree_contents([('debian/', ), ('debian/control', """\
Source: blah
Testsuite: autopkgtest

""")])

        def add_header(control):
            control["XS-Vcs-Git"] = "git://github.com/example/example"
        self.assertTrue(update_control(source_package_cb=add_header))
        self.assertFileEqual("""\
Source: blah
Testsuite: autopkgtest
XS-Vcs-Git: git://github.com/example/example
""", 'debian/control')

    def test_modify_binary(self):
        self.build_tree_contents([('debian/', ), ('debian/control', """\
Source: blah
Testsuite: autopkgtest

Package: libblah
Section: extra

""")])

        def add_header(control):
            control["Arch"] = "all"
        self.assertTrue(update_control(binary_package_cb=add_header))
        self.assertFileEqual("""\
Source: blah
Testsuite: autopkgtest

Package: libblah
Section: extra
Arch: all
""", 'debian/control')

    def test_doesnt_strip_whitespace(self):
        self.build_tree_contents([('debian/', ), ('debian/control', """\
Source: blah
Testsuite: autopkgtest

""")])
        self.assertFalse(update_control())
        self.assertFileEqual("""\
Source: blah
Testsuite: autopkgtest

""", 'debian/control')

    def test_update_template(self):
        self.build_tree_contents([('debian/', ), ('debian/control', """\
# DO NOT EDIT
# This file was generated by blah

Source: blah
Testsuite: autopkgtest
Uploaders: Jelmer Vernooij <jelmer@jelmer.uk>

"""), ('debian/control.in', """\
Source: blah
Testsuite: autopkgtest
Uploaders: @lintian-brush-test@

""")])

        def source_cb(c):
            c['Testsuite'] = 'autopkgtest8'
        with ControlUpdater() as updater:
            source_cb(updater.source)
            updater.changes()
        self.assertFileEqual("""\
Source: blah
Testsuite: autopkgtest8
Uploaders: @lintian-brush-test@
""", "debian/control.in")
        self.assertFileEqual("""\
Source: blah
Testsuite: autopkgtest8
Uploaders: testvalue
""", "debian/control")

    def test_description_stays_last(self):
        self.build_tree_contents([('debian/', ), ('debian/control', """\
Source: blah
Testsuite: autopkgtest

Package: libblah
Section: extra
Description: foo
 bar

""")])

        def add_header(control):
            control["Arch"] = "all"
        self.assertTrue(update_control(binary_package_cb=add_header))
        self.assertFileEqual("""\
Source: blah
Testsuite: autopkgtest

Package: libblah
Section: extra
Arch: all
Description: foo
 bar
""", 'debian/control')


class ParseRelationsTests(TestCase):

    def test_empty(self):
        self.assertEqual([], parse_relations(''))
        self.assertEqual([('\n', [], '')], parse_relations('\n'))

    def test_simple(self):
        self.assertEqual(
                [('', [PkgRelation('debhelper')], '')],
                parse_relations('debhelper'))
        self.assertEqual(
                [('  \n', [PkgRelation('debhelper')], '')],
                parse_relations('  \ndebhelper'))
        self.assertEqual(
                [('  \n', [PkgRelation('debhelper')], ' \n')],
                parse_relations('  \ndebhelper \n'))


class FormatRelationsTests(TestCase):

    def test_empty(self):
        self.assertEqual(
                '',
                format_relations([('', [], '')]))
        self.assertEqual(
                '\n',
                format_relations([('', [], '\n')]))

    def test_simple(self):
        self.assertEqual(
                'debhelper',
                format_relations([('', [PkgRelation('debhelper')], '')]))
        self.assertEqual(
                format_relations([('  \n', [PkgRelation('debhelper')], '')]),
                '  \ndebhelper')
        self.assertEqual(
                format_relations(
                    [('  \n', [PkgRelation('debhelper')], ' \n')]),
                '  \ndebhelper \n')

    def test_multiple(self):
        self.assertEqual(
                'debhelper, blah',
                format_relations([('', [PkgRelation('debhelper')], ''),
                                 (' ', [PkgRelation('blah')], '')]))


class EnsureMinimumVersionTests(TestCase):

    def test_added(self):
        self.assertEqual(
            'debhelper (>= 9)', ensure_minimum_version('', 'debhelper', '9'))
        self.assertEqual(
            'blah, debhelper (>= 9)',
            ensure_minimum_version('blah', 'debhelper', '9'))

    def test_unchanged(self):
        self.assertEqual(
            'debhelper (>= 9)', ensure_minimum_version(
                'debhelper (>= 9)', 'debhelper', '9'))
        self.assertEqual(
            'debhelper (= 9)', ensure_minimum_version(
                'debhelper (= 9)', 'debhelper', '9'))
        self.assertEqual(
            'debhelper (>= 9)', ensure_minimum_version(
                'debhelper (>= 9)', 'debhelper', '9~'))

    def test_updated(self):
        self.assertEqual(
            'debhelper (>= 9)',
            ensure_minimum_version('debhelper', 'debhelper', '9'))
        self.assertEqual(
            'blah, debhelper (>= 9)',
            ensure_minimum_version('blah, debhelper', 'debhelper', '9'))
        self.assertEqual(
            'blah, debhelper (>= 9)',
            ensure_minimum_version('blah, debhelper (>= 8)', 'debhelper', '9'))
        self.assertEqual(
            'blah, debhelper (>= 9)',
            ensure_minimum_version(
                'blah, debhelper (>= 8), debhelper (>= 8.1) | dh-systemd',
                'debhelper', '9'))
        self.assertEqual(
            'blah, debhelper (>= 9), debhelper (>= 10) | dh-systemd',
            ensure_minimum_version(
                'blah, debhelper (>= 8), debhelper (>= 10) | dh-systemd',
                'debhelper', '9'))


class EnsureSomeVersionTests(TestCase):

    def test_added(self):
        self.assertEqual(
            'debhelper', ensure_some_version('', 'debhelper'))
        self.assertEqual(
            'blah, debhelper',
            ensure_some_version('blah', 'debhelper'))

    def test_unchanged(self):
        self.assertEqual(
            'debhelper (>= 9)', ensure_some_version(
                'debhelper (>= 9)', 'debhelper'))
        self.assertEqual(
            'debhelper (= 9)', ensure_some_version(
                'debhelper (= 9)', 'debhelper'))
        self.assertEqual(
            'debhelper (>= 9)', ensure_some_version(
                'debhelper (>= 9)', 'debhelper'))
        self.assertEqual(
            'debhelper', ensure_some_version(
                'debhelper', 'debhelper'))


class EnsureExactVersionTests(TestCase):

    def test_added(self):
        self.assertEqual(
            'debhelper (= 9)', ensure_exact_version('', 'debhelper', '9'))
        self.assertEqual(
            'blah, debhelper (= 9)',
            ensure_exact_version('blah', 'debhelper', '9'))

    def test_unchanged(self):
        self.assertEqual(
            'debhelper (= 9)', ensure_exact_version(
                'debhelper (= 9)', 'debhelper', '9'))

    def test_updated(self):
        self.assertEqual(
            'debhelper (= 9)', ensure_exact_version(
                'debhelper (>= 9)', 'debhelper', '9'))
        self.assertEqual(
            'debhelper (= 9)',
            ensure_exact_version('debhelper', 'debhelper', '9'))
        self.assertEqual(
            'blah, debhelper (= 9)',
            ensure_exact_version('blah, debhelper', 'debhelper', '9'))
        self.assertEqual(
            'blah, debhelper (= 9)',
            ensure_exact_version('blah, debhelper (= 8)', 'debhelper', '9'))
        self.assertEqual(
            'blah, debhelper (= 9)',
            ensure_exact_version('blah, debhelper (= 10)', 'debhelper', '9'))


class DropDependencyTests(TestCase):

    def test_deleted(self):
        self.assertEqual(
            'debhelper (>= 9)',
            drop_dependency('debhelper (>= 9), dh-autoreconf',
                            'dh-autoreconf'))
        self.assertEqual(
            'dh-autoreconf',
            drop_dependency('debhelper (>= 9), dh-autoreconf', 'debhelper'))
        self.assertEqual(
            '', drop_dependency('debhelper (>= 9)', 'debhelper'))
        self.assertEqual(
            'debhelper-compat (= 12)',
            drop_dependency(
                'debhelper (>= 9), debhelper-compat (= 12)',
                'debhelper'))


class AddDependencyTests(TestCase):

    def test_added(self):
        self.assertEqual(
            'debhelper (>= 9), dh-autoreconf',
            add_dependency('debhelper (>= 9)', 'dh-autoreconf'))
        self.assertEqual(
            'debhelper (>= 9), ${misc:Depends}',
            add_dependency('debhelper (>= 9)', '${misc:Depends}'))
        self.assertEqual(
            'debhelper (>= 9), blah,',
            add_dependency('debhelper (>= 9),', 'blah'))

    def test_indentation(self):
        self.assertEqual("""foo,
    bar,
    blah""", add_dependency("""foo,
    bar""", 'blah'))
        self.assertEqual("""foo,
 bar,
 blah""", add_dependency("""foo,
 bar""", 'blah'))
        self.assertEqual("""foo,
 bar,
 blah
""", add_dependency("""foo,
 bar
""", 'blah'))

    def test_insert(self):
        self.assertEqual("""blah,
    foo,
    bar""", add_dependency("""foo,
    bar""", 'blah', position=0))
        self.assertEqual("""foo,
    blah,
    bar""", add_dependency("""foo,
    bar""", 'blah', position=1))

    def test_odd_syntax(self):
        self.assertEqual("""
 foo
 , bar
 , blah
""", add_dependency("""
 foo
 , bar
""", 'blah'))
        self.assertEqual("""
 foo
 , blah
 , bar
""", add_dependency("""
 foo
 , bar
""", 'blah', position=1))


class GetRelationTests(TestCase):

    def test_missing(self):
        self.assertRaises(
            KeyError, get_relation, '', 'debhelper')
        self.assertRaises(
            KeyError,
            get_relation, 'blah', 'debhelper')

    def test_simple(self):
        self.assertEqual(
            (0, [PkgRelation('debhelper', ('>=', '9'))]),
            get_relation(
                'debhelper (>= 9)', 'debhelper'))
        self.assertEqual(
            (1, [PkgRelation('debhelper', ('=', '9'))]),
            get_relation(
                'blah, debhelper (= 9)', 'debhelper'))

    def test_complex(self):
        self.assertRaises(
            ValueError,
            get_relation, 'blah | debhelper (= 9)', 'debhelper')
        self.assertRaises(
            ValueError,
            get_relation,
            'blah, debhelper (= 9) | debhelper (<< 10)', 'debhelper')


class IterRelationsTests(TestCase):

    def test_missing(self):
        self.assertEqual(
            [], list(iter_relations('', 'debhelper')))
        self.assertEqual(
            [], list(iter_relations('blah', 'debhelper')))

    def test_simple(self):
        self.assertEqual(
            [(0, [PkgRelation('debhelper', ('>=', '9'))])],
            list(iter_relations(
                'debhelper (>= 9)', 'debhelper')))
        self.assertEqual(
            [(1, [PkgRelation('debhelper', ('=', '9'))])],
            list(iter_relations(
                'blah, debhelper (= 9)', 'debhelper')))

    def test_complex(self):
        self.assertEqual(
            [(0, [PkgRelation('blah'), PkgRelation('debhelper', ('=', '9'))])],
            list(iter_relations('blah | debhelper (= 9)', 'debhelper')))
        self.assertEqual(
            [(1,
              [PkgRelation('debhelper', ('=', '9')),
               PkgRelation('debhelper', ('<<', '10'))])],
            list(iter_relations(
                'blah, debhelper (= 9) | debhelper (<< 10)', 'debhelper')))


class DeleteFromListTests(TestCase):

    def test_intermediate(self):
        self.assertEqual('a, c', delete_from_list('a, b, c', 'b'))
        self.assertEqual('a, c', delete_from_list('a, b, c', 'b '))

    def test_head(self):
        self.assertEqual('b, c', delete_from_list('a, b, c', 'a'))
        self.assertEqual(' b, c', delete_from_list(' a, b, c', 'a'))

    def test_tail(self):
        self.assertEqual('a, b', delete_from_list('a, b, c', 'c'))
        self.assertEqual('a, b', delete_from_list('a, b , c', 'c'))

    def test_only(self):
        self.assertEqual('a', delete_from_list('a', 'c'))
        self.assertEqual('', delete_from_list('a', 'a'))


class IsDepImpliedTests(TestCase):

    def parse(self, p):
        [dep] = PkgRelation.parse(p)
        return dep

    def test_no_version(self):
        self.assertTrue(
            is_dep_implied(self.parse('bzr'), self.parse('bzr')))
        self.assertTrue(
            is_dep_implied(self.parse('bzr'), self.parse('bzr (>= 3)')))
        self.assertTrue(
            is_dep_implied(self.parse('bzr'), self.parse('bzr (<< 3)')))

    def test_wrong_package(self):
        self.assertFalse(
            is_dep_implied(self.parse('bzr'), self.parse('foo (<< 3)')))

    def test_version(self):
        self.assertFalse(
            is_dep_implied(self.parse('bzr (>= 3)'), self.parse('bzr (<< 3)')))
        self.assertTrue(
            is_dep_implied(self.parse('bzr (>= 3)'), self.parse('bzr (= 3)')))
        self.assertFalse(
            is_dep_implied(self.parse('bzr (= 3)'), self.parse('bzr (>= 3)')))
        self.assertFalse(
            is_dep_implied(self.parse('bzr (>= 3)'), self.parse('bzr (>> 3)')))
        self.assertFalse(
            is_dep_implied(self.parse('bzr (= 3)'), self.parse('bzr (= 4)')))
        self.assertFalse(
            is_dep_implied(self.parse('bzr (>= 3)'), self.parse('bzr (>= 2)')))
        self.assertTrue(
            is_dep_implied(self.parse('bzr (>= 3)'), self.parse('bzr (>= 3)')))


class IsRelationImpliedTests(TestCase):

    def test_unrelated(self):
        self.assertFalse(is_relation_implied('bzr', 'bar'))
        self.assertFalse(is_relation_implied('bzr (= 3)', 'bar'))
        self.assertFalse(is_relation_implied('bzr (= 3) | foo', 'bar'))

    def test_too_old(self):
        self.assertFalse(is_relation_implied('bzr (= 3)', 'bzr'))
        self.assertFalse(is_relation_implied('bzr (= 3)', 'bzr (= 2)'))
        self.assertFalse(is_relation_implied('bzr (= 3)', 'bzr (>= 2)'))

    def test_ors(self):
        self.assertFalse(is_relation_implied('bzr (= 3)', 'bzr | foo'))
        self.assertFalse(is_relation_implied('bzr', 'bzr | foo'))
        self.assertTrue(is_relation_implied('bzr | foo', 'bzr | foo'))

    def test_implied(self):
        self.assertTrue(is_relation_implied('bzr (= 3)', 'bzr (= 3)'))
        self.assertTrue(is_relation_implied('bzr', 'bzr'))
        self.assertTrue(is_relation_implied('bzr | foo', 'bzr'))
        self.assertFalse(is_relation_implied('bzr (= 3)', 'bzr (>= 3)'))



#!/usr/bin/python3
# Copyright (C) 2018 Jelmer Vernooij <jelmer@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import argparse
import atexit
import shutil
import sys
import tempfile

import distro_info

from breezy.branch import Branch
from breezy.workingtree import WorkingTree
import locale
locale.setlocale(locale.LC_ALL, '')
# Use better default than ascii with posix filesystems that deal in bytes
# natively even when the C locale or no locale at all is given. Note that
# we need an immortal string for the hack, hence the lack of a hyphen.
sys._brz_default_fs_enc = "utf8"

import breezy  # noqa: E402
from breezy.errors import (
    DependencyNotPresent,  # noqa: E402
    NotBranchError,
    )
breezy.initialize()
import breezy.git  # noqa: E402
import breezy.bzr  # noqa: E402

from breezy.trace import note, show_error  # noqa: E402

from . import (  # noqa: E402
    NotDebianPackage,
    PendingChanges,
    available_lintian_fixers,
    find_fixers_dir,
    get_committer,
    run_lintian_fixers,
    select_fixers,
    version_string,
    SUPPORTED_CERTAINTIES,
    DEFAULT_MINIMUM_CERTAINTY,
    )
from .config import Config   # noqa: E402


def main(argv=None):
    parser = argparse.ArgumentParser(prog='lintian-brush')
    parser.add_argument(
        '--no-update-changelog', action="store_false", default=None,
        dest="update_changelog", help="do not update the changelog")
    parser.add_argument(
        '--update-changelog', action="store_true", dest="update_changelog",
        help="force updating of the changelog", default=None)
    parser.add_argument(
        '--version', action='version', version='%(prog)s ' + version_string)
    parser.add_argument(
        '--list-fixers', action="store_true", help="list available fixers")
    parser.add_argument(
        '--list-tags', action="store_true",
        help="list lintian tags for which fixers are available")
    parser.add_argument(
        '--fixers-dir', type=str, help='path to fixer scripts. [%(default)s]',
        default=find_fixers_dir())
    parser.add_argument(
        '--verbose', help='be verbose', action='store_true', default=False)
    parser.add_argument(
        '--directory', metavar='DIRECTORY', help='directory to run in',
        type=str, default='.')
    parser.add_argument(
        '--diff', help='Print resulting diff afterwards.', action='store_true')
    parser.add_argument(
        '--dry-run', help=(
            'Do not make any changes to the current repository. '
            'Note: currently creates a temporary clone of the repository.'),
        action='store_true')
    parser.add_argument(
        '--modern', help=(
            'Use features/compatibility levels that are not available in '
            'stable. (makes backporting harder)'),
        action='store_true', default=False)
    parser.add_argument(
        '--identity',
        help='Print user identity that would be used when committing',
        action='store_true', default=False)
    parser.add_argument(
        '--disable-net-access',
        help='Do not probe external services.',
        action='store_true', default=False)
    # Hide the minimum-certainty option for the moment.
    parser.add_argument(
        '--minimum-certainty',
        type=str,
        choices=SUPPORTED_CERTAINTIES,
        default=None,
        help=argparse.SUPPRESS)
    parser.add_argument(
        '--trust',
        action='store_true',
        help=argparse.SUPPRESS)
    parser.add_argument(
        '--allow-reformatting', default=None, action='store_true',
        help=argparse.SUPPRESS)
    parser.add_argument(
        '--disable-inotify', action='store_true', default=False,
        help=argparse.SUPPRESS)
    parser.add_argument('--compat-release', type=str, help=argparse.SUPPRESS)
    parser.add_argument(
        '--exclude', metavar='EXCLUDE', type=str, action='append',
        help='Exclude a fixer.')
    parser.add_argument(
        '--opinionated', action='store_true',
        help=argparse.SUPPRESS)
    parser.add_argument(
        '--diligent', action='count', default=0, dest='diligence',
        help=argparse.SUPPRESS)
    parser.add_argument(
        'fixers', metavar='FIXER', nargs='*',
        help='specific fixer to run')
    args = parser.parse_args(argv)

    if args.list_fixers and args.list_tags:
        parser.print_usage()
        return 1

    fixers = available_lintian_fixers(args.fixers_dir)
    if args.list_fixers:
        for script in sorted([fixer.name for fixer in fixers]):
            note(script)
    elif args.list_tags:
        tags = set()
        for fixer in fixers:
            tags.update(fixer.lintian_tags)
        for tag in sorted(tags):
            note(tag)
    else:
        try:
            if args.dry_run:
                branch, subpath = Branch.open_containing(args.directory)
                td = tempfile.mkdtemp()
                atexit.register(shutil.rmtree, td)
                # TODO(jelmer): Make a slimmer copy
                to_dir = branch.controldir.sprout(
                    td, None, create_tree_if_local=True,
                    source_branch=branch,
                    stacked=branch._format.supports_stacking())
                wt = to_dir.open_workingtree()
            else:
                wt, subpath = WorkingTree.open_containing(args.directory)
        except NotBranchError:
            note('No version control directory found (e.g. a .git directory).')
            return 1
        except DependencyNotPresent as e:
            note('Unable to open tree at %s: missing package %s',
                 args.directory, e.library)
            return 1
        if args.identity:
            print(get_committer(wt))
            return 0
        since_revid = wt.last_revision()
        if args.fixers:
            try:
                fixers = select_fixers(fixers, args.fixers, args.exclude)
            except KeyError as e:
                show_error('Unknown fixer specified: %s', e.args[0])
                return 1
        debian_info = distro_info.DebianDistroInfo()
        if args.modern:
            if args.compat_release:
                show_error('--compat-release and --modern are incompatible.')
                return 1
            compat_release = debian_info.devel()
        else:
            compat_release = args.compat_release
        minimum_certainty = args.minimum_certainty
        allow_reformatting = args.allow_reformatting
        update_changelog = args.update_changelog
        try:
            cfg = Config.from_workingtree(wt, subpath)
        except FileNotFoundError:
            pass
        else:
            if minimum_certainty is None:
                minimum_certainty = cfg.minimum_certainty()
            if compat_release is None:
                compat_release = cfg.compat_release()
            if allow_reformatting is None:
                allow_reformatting = cfg.allow_reformatting()
            if update_changelog is None:
                update_changelog = cfg.update_changelog()
        if minimum_certainty is None:
            minimum_certainty = DEFAULT_MINIMUM_CERTAINTY
        if compat_release is None:
            compat_release = debian_info.stable()
        if allow_reformatting is None:
            allow_reformatting = False
        with wt.lock_write():
            try:
                overall_result = run_lintian_fixers(
                    wt, fixers,
                    update_changelog=update_changelog,
                    compat_release=compat_release,
                    verbose=args.verbose,
                    minimum_certainty=args.minimum_certainty,
                    trust_package=args.trust,
                    allow_reformatting=allow_reformatting,
                    use_inotify=(False if args.disable_inotify else None),
                    subpath=subpath,
                    net_access=not args.disable_net_access,
                    opinionated=args.opinionated,
                    diligence=args.diligence)
            except NotDebianPackage:
                note("%s: Not a debian package.", wt.basedir)
                return 1
            except PendingChanges:
                note("%s: Please commit pending changes first.", wt.basedir)
                if args.verbose:
                    from breezy.status import show_tree_status
                    show_tree_status(wt)
                return 1
        if overall_result.success:
            all_tags = set()
            for result, summary in overall_result.success:
                all_tags.update(result.fixed_lintian_tags)
            if all_tags:
                note("Lintian tags fixed: %r" % all_tags)
            else:
                note("Some changes were made, "
                     "but there are no affected lintian tags.")
        else:
            note("No changes made.")
        if overall_result.failed_fixers and not args.verbose:
            note("Some fixer scripts failed to run: %r. "
                 "Run with --verbose for details.",
                 set(overall_result.failed_fixers.keys()))
        if overall_result.formatting_unpreservable and not args.verbose:
            note('Some fixer scripts were unable to preserve formatting: %r. '
                 'Run with --allow-reformatting to reformat %r.',
                 set(overall_result.formatting_unpreservable),
                 set(overall_result.formatting_unpreservable.values()))
        if args.diff:
            from breezy.diff import show_diff_trees
            show_diff_trees(
                wt.branch.repository.revision_tree(since_revid),
                wt, sys.stdout.buffer)


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/python3
# Copyright (C) 2018 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Tests for the vcs module."""

from unittest import TestCase

from lintian_brush.vcs import (
    canonicalize_vcs_url,
    determine_browser_url,
    fixup_broken_git_url,
    plausible_url,
    sanitize_url,
    split_vcs_url,
    unsplit_vcs_url,
    )


class FixUpGitUrlTests(TestCase):

    def test_fixup(self):
        self.assertEqual(
            'git://github.com/jelmer/dulwich',
            fixup_broken_git_url('git://github.com:jelmer/dulwich'))
        self.assertEqual(
            'git://github.com/jelmer/dulwich -b blah',
            fixup_broken_git_url('git://github.com:jelmer/dulwich -b blah'))

    def test_preserves(self):
        self.assertEqual(
            'git://github.com/jelmer/dulwich',
            fixup_broken_git_url('git://github.com/jelmer/dulwich'))
        self.assertEqual(
            'https://github.com/jelmer/dulwich',
            fixup_broken_git_url('https://github.com/jelmer/dulwich'))

    def test_salsa_not_https(self):
        self.assertEqual(
            'https://salsa.debian.org/jelmer/dulwich',
            fixup_broken_git_url(
                'git://salsa.debian.org/jelmer/dulwich'))

    def test_salsa_uses_cgit(self):
        self.assertEqual(
            'https://salsa.debian.org/jelmer/dulwich',
            fixup_broken_git_url(
                'https://salsa.debian.org/cgit/jelmer/dulwich'))

    def test_salsa_tree_branch(self):
        self.assertEqual(
            'https://salsa.debian.org/jelmer/dulwich -b master',
            fixup_broken_git_url(
                'https://salsa.debian.org/jelmer/dulwich/tree/master'))

    def test_strip_extra_slash(self):
        self.assertEqual(
            'https://salsa.debian.org/salve/auctex.git',
            fixup_broken_git_url('https://salsa.debian.org//salve/auctex.git'))

    def test_strip_extra_colon(self):
        self.assertEqual(
            'https://salsa.debian.org/mckinstry/lcov.git',
            fixup_broken_git_url(
                'https://salsa.debian.org:/mckinstry/lcov.git'))

    def test_strip_username(self):
        self.assertEqual(
            'git://github.com/RPi-Distro/pgzero.git',
            fixup_broken_git_url('git://git@github.com:RPi-Distro/pgzero.git'))
        self.assertEqual(
            'https://salsa.debian.org/debian-astro-team/pyavm.git',
            fixup_broken_git_url(
                'https://git@salsa.debian.org:debian-astro-team/pyavm.git'))

    def test_github_tree_url(self):
        self.assertEqual(
            'https://github.com/blah/blah -b master',
            fixup_broken_git_url('https://github.com/blah/blah/tree/master'))

    def test_freedesktop(self):
        self.assertEqual(
            'https://gitlab.freedesktop.org/xorg/xserver',
            fixup_broken_git_url('git://anongit.freedesktop.org/xorg/xserver'))

    def test_gnome(self):
        self.assertEqual(
            'https://gitlab.gnome.org/GNOME/alacarte',
            fixup_broken_git_url('https://git.gnome.org/browse/alacarte'))


class SanitizeUrlTests(TestCase):

    def test_simple(self):
        self.assertEqual(
            'https://github.com/jelmer/blah.git',
            sanitize_url('http://github.com/jelmer/blah'))

    def test_git_http(self):
        self.assertEqual(
            'https://github.com/jelmer/blah.git',
            sanitize_url('git+http://github.com/jelmer/blah'))
        self.assertEqual(
            'https://github.com/jelmer/blah.git',
            sanitize_url('git+https://github.com/jelmer/blah'))


class DetermineBrowserUrlTests(TestCase):

    def test_salsa(self):
        self.assertEqual(
            'https://salsa.debian.org/jelmer/dulwich',
            determine_browser_url(
                'git', 'https://salsa.debian.org/jelmer/dulwich.git'))

    def test_github(self):
        self.assertEqual(
            'https://github.com/jelmer/dulwich',
            determine_browser_url(
                'git', 'https://github.com/jelmer/dulwich.git'))
        self.assertEqual(
            'https://github.com/jelmer/dulwich/tree/master',
            determine_browser_url(
                'git', 'https://github.com/jelmer/dulwich.git -b master'))
        self.assertEqual(
            'https://github.com/jelmer/dulwich/tree/master',
            determine_browser_url(
                'git', 'git://github.com/jelmer/dulwich -b master'))
        self.assertEqual(
            'https://github.com/jelmer/dulwich/tree/master/blah',
            determine_browser_url(
                'git', 'git://github.com/jelmer/dulwich -b master [blah]'))
        self.assertEqual(
            'https://github.com/jelmer/dulwich/tree/HEAD/blah',
            determine_browser_url(
                'git', 'git://github.com/jelmer/dulwich [blah]'))
        self.assertEqual(
            'https://git.sv.gnu.org/cgit/rcs.git',
            determine_browser_url(
                'git', 'https://git.sv.gnu.org/git/rcs.git'))
        self.assertEqual(
            'https://git.savannah.gnu.org/cgit/rcs.git',
            determine_browser_url(
                'git', 'git://git.savannah.gnu.org/rcs.git'))


class PlausibleUrlTests(TestCase):

    def test_url(self):
        self.assertFalse(plausible_url('the'))
        self.assertFalse(plausible_url('1'))
        self.assertTrue(plausible_url('git@foo:blah'))
        self.assertTrue(plausible_url('git+ssh://git@foo/blah'))
        self.assertTrue(plausible_url('https://foo/blah'))


class SplitVcsUrlTests(TestCase):

    def test_none(self):
        self.assertEqual(
            ('https://github.com/jelmer/example', None, None),
            split_vcs_url('https://github.com/jelmer/example'))
        self.assertEqual(
            ('https://github.com/jelmer/example', None, 'path/to/packaging'),
            split_vcs_url(
                'https://github.com/jelmer/example [path/to/packaging]'))

    def test_branch(self):
        self.assertEqual(
            ('https://github.com/jelmer/example',
                'master', 'path/to/packaging'),
            split_vcs_url(
                'https://github.com/jelmer/example [path/to/packaging] '
                '-b master'))
        self.assertEqual(
            ('https://github.com/jelmer/example',
                'master', 'path/to/packaging'),
            split_vcs_url(
                'https://github.com/jelmer/example -b master '
                '[path/to/packaging]'))
        self.assertEqual(
            ('https://github.com/jelmer/example', 'master', None),
            split_vcs_url(
                'https://github.com/jelmer/example -b master'))


class UnsplitVcsUrlTests(TestCase):

    def test_none(self):
        self.assertEqual(
            'https://github.com/jelmer/example',
            unsplit_vcs_url('https://github.com/jelmer/example', None, None))
        self.assertEqual(
            'https://github.com/jelmer/example [path/to/packaging]',
            unsplit_vcs_url(
                'https://github.com/jelmer/example', None,
                'path/to/packaging'))

    def test_branch(self):
        self.assertEqual(
            'https://github.com/jelmer/example -b master '
            '[path/to/packaging]',
            unsplit_vcs_url(
                'https://github.com/jelmer/example', 'master',
                'path/to/packaging'))
        self.assertEqual(
            'https://github.com/jelmer/example -b master',
            unsplit_vcs_url(
                'https://github.com/jelmer/example', 'master', None))


class CanonicalizeVcsUrlTests(TestCase):

    def test_github(self):
        self.assertEqual(
            'https://github.com/jelmer/example.git',
            canonicalize_vcs_url('Git', 'https://github.com/jelmer/example'))

    def test_salsa(self):
        self.assertEqual(
            'https://salsa.debian.org/jelmer/example.git',
            canonicalize_vcs_url(
                'Git', 'https://salsa.debian.org/jelmer/example'))
        self.assertEqual(
            'https://salsa.debian.org/jelmer/example.git',
            canonicalize_vcs_url(
                'Git', 'https://salsa.debian.org/jelmer/example.git'))



#!/usr/bin/python3

import os
try:
    with open('debian/source/options', 'r') as f:
        if not f.read().strip():
            os.unlink('debian/source/options')
except FileNotFoundError:
    pass

print('Remove empty debian/source/options.')



#!/usr/bin/python3

from lintian_brush.xdg import DesktopEntryUpdater

import os

paths = []

for name in os.listdir('debian'):
    if name.endswith('.desktop'):
        path = os.path.join('debian', name)
        with DesktopEntryUpdater(path) as updater:
            if updater.get('Encoding') == 'UTF-8':
                del updater.entry['Encoding']
                paths.append(path)
            # TODO(jelmer): if encoding is non-UTF-8, invoke iconv.


if len(paths) == 1:
    print('Remove deprecated Encoding key from desktop file %s.' %
          paths[0])
else:
    print('Remove deprecated Encoding key from desktop files: %s.' % (
          ', '.join(sorted(paths))))
print('Fixed-Lintian-Tags: desktop-entry-contains-encoding-key')



#!/usr/bin/python3
from lintian_brush.control import ControlUpdater
from lintian_brush.fixer import report_result
fields = []
packages = []


with ControlUpdater() as updater:
    for para in updater.paragraphs:
        for k, v in para.items():
            if not v.strip():
                fields.append(k)
                if para.get("Package"):
                    packages.append(para.get("Package"))
                del para[k]

report_result(
    "debian/control: Remove empty control field%s %s%s." % (
     "s" if len(fields) > 1 else "",
     ", ".join(fields),
     (" in package %s" % ', '.join(packages)) if packages else "",
    ),
    fixed_lintian_tags=['debian-control-has-empty-field'])



#!/usr/bin/python3

import os
from debian.changelog import Changelog
from lintian_brush.control import ControlUpdater, get_relation
from debian.copyright import Copyright, NotMachineReadableError
from debian.deb822 import Deb822

# Dictionary mapping source and target versions
upgrade_path = {
    "4.1.0": "4.1.1",
    "4.2.0": "4.2.1",
    "4.3.0": "4.4.0",
    "4.4.0": "4.4.1",
    "4.1.4": "4.1.5",
}


def check_4_1_1():
    return os.path.exists("debian/changelog")


def check_4_4_0():
    # Check that the package uses debhelper.
    if os.path.exists("debian/compat"):
        return True
    with open('debian/control') as f:
        source = next(Deb822.iter_paragraphs(f))
        build_deps = source.get('Build-Depends', '')
        try:
            get_relation(build_deps, 'debhelper-compat')
        except KeyError:
            return False
        else:
            return True


def check_4_4_1():
    # Check that there is only one Vcs field.
    vcs_fields = []
    with open('debian/control') as f:
        source = next(Deb822.iter_paragraphs(f))
        for name in source:
            if name.lower() == 'vcs-browser':
                continue
            if name.lower().startswith('vcs-'):
                vcs_fields.append(name)
    if len(vcs_fields) > 1:
        return False

    # Check that Files entries don't refer to directories.
    # They must be wildcards *in* the directories.
    try:
        with open('debian/copyright', 'r') as f:
            copyright = Copyright(f)
            for para in copyright.all_files_paragraphs():
                for glob in para.files:
                    if os.path.isdir(glob):
                        return False
    except FileNotFoundError:
        return False
    except NotMachineReadableError:
        pass
    return True


def check_4_1_5():
    # If epoch has changed -> return False
    with open('debian/changelog', 'r') as f:
        cl = Changelog(f, max_blocks=2)
        epochs = set()
        for block in cl:
            epochs.add(block.version.epoch)
        if len(epochs) > 1:
            return False

    with open('debian/control') as f:
        source = Deb822(f)
        if 'Rules-Requires-Root' not in source:
            return False

    return True


check_requirements = {
    "4.1.1": check_4_1_1,
    "4.4.0": check_4_4_0,
    "4.4.1": check_4_4_1,
    "4.1.5": check_4_1_5,
}

current_version = None


with ControlUpdater() as updater:
    try:
        current_version = updater.source["Standards-Version"]
    except KeyError:
        # Huh, no standards version?
        pass
    else:
        while current_version in upgrade_path:
            target_version = upgrade_path[current_version]
            try:
                check_fn = check_requirements[target_version]
            except KeyError:
                pass
            else:
                if not check_fn():
                    break
            current_version = target_version
        updater.source["Standards-Version"] = current_version


if current_version:
    print('Update standards version to %s, no changes needed.' %
          current_version)
print('Certainty: certain')
print('Fixed-Lintian-Tags: out-of-date-standards-version')



#!/usr/bin/python3
import os

try:
    st = os.stat('debian/rules')
except FileNotFoundError:
    pass
else:
    if not (st.st_mode & 0o111):
        os.chmod('debian/rules', 0o755)


print('Make debian/rules executable.')
print('Fixed-Lintian-Tags: debian-rules-not-executable')



#!/usr/bin/python3

from lintian_brush.control import update_control
from lintian_brush.fixer import net_access_allowed
from lintian_brush.vcs import find_secure_vcs_url

updated = set()
lp_note = False


def fix_insecure_vcs_uri(control):
    global updated, lp_note
    for key, value in control.items():
        if not key.startswith('Vcs-'):
            continue
        if value.startswith('lp:'):
            lp_note = True
        newvalue = find_secure_vcs_url(
            value, net_access=net_access_allowed())
        if newvalue is None:
            # We can't find a secure version :(
            continue
        if newvalue == value:
            # The URL was already secure
            continue
        control[key] = newvalue
        updated.add(key)


update_control(source_package_cb=fix_insecure_vcs_uri)

if len(updated) == 1:
    print("Use secure URI in Vcs control header %s." % list(updated)[0])
else:
    print("Use secure URI in Vcs control headers: %s." %
          ', '.join(sorted(updated)))
if lp_note:
    print("")
    print("The lp: prefix gets expanded to http://code.launchpad.net/ for "
          "users that are not logged in on some versions of Bazaar.")
print("Fixed-Lintian-Tags: vcs-field-uses-insecure-uri")



#!/usr/bin/python
# Copyright (C) 2018 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Tests for lintian_brush.lintian_overrides."""

from breezy.tests import (
    TestCase,
    TestCaseWithTransport,
    )

from lintian_brush.lintian_overrides import (
    Override,
    overrides_paths,
    override_exists,
    parse_override,
    serialize_override,
    update_overrides_file,
    _get_overrides,
    )


class OverridesPathTests(TestCaseWithTransport):

    def test_no_overrides_paths(self):
        self.assertEqual([], list(overrides_paths()))

    def test_overrides_paths(self):
        self.build_tree(
            ['debian/', 'debian/source/', 'debian/source/lintian-overrides'])
        self.assertEqual(
            ['debian/source/lintian-overrides'],
            list(overrides_paths()))


class UpdateOverridesFileTests(TestCaseWithTransport):

    def test_no_changes(self):
        CONTENT = """\
# An architecture wildcard would look like:
foo [any-i386] binary: another-tag optional-extra
"""
        self.build_tree_contents([('overrides', CONTENT)])

        def cb(override):
            return override

        self.assertFalse(update_overrides_file(cb=cb, path='overrides'))
        self.assertFileEqual(CONTENT, 'overrides')

    def test_change_set_archlist(self):
        self.build_tree_contents([('overrides', """\
# An architecture wildcard would look like:
foo binary: another-tag optional-extra
""")])

        def cb(override):
            return Override(
                tag=override.tag, package=override.package,
                type=override.type, info=override.info,
                archlist='any-i386')

        self.assertTrue(update_overrides_file(cb=cb, path='overrides'))
        self.assertFileEqual("""\
# An architecture wildcard would look like:
foo [any-i386] binary: another-tag optional-extra
""", 'overrides')


class ParseOverrideTests(TestCase):

    def test_tag_only(self):
        self.assertEqual(
            Override(tag='sometag'),
            parse_override('sometag\n'))

    def test_origin(self):
        self.assertEqual(
            Override(tag='sometag', package='mypkg'),
            parse_override('mypkg: sometag\n'))

    def test_archlist(self):
        self.assertEqual(
            Override(tag='sometag', archlist='i386 amd64'),
            parse_override('[i386 amd64]: sometag\n'))


class SerializeOverrideTests(TestCase):

    def test_tag_only(self):
        self.assertEqual(
            serialize_override(Override(tag='sometag')),
            'sometag\n')

    def test_origin(self):
        self.assertEqual(
            serialize_override(Override(tag='sometag', package='mypkg')),
            'mypkg: sometag\n')

    def test_archlist(self):
        self.assertEqual(
            serialize_override(Override(tag='sometag', archlist='i386')),
            '[i386]: sometag\n')


class OverrideExistsTests(TestCaseWithTransport):

    def test_override_exists(self):
        self.build_tree_contents([
            ('debian/', ),
            ('debian/source/', ),
            ('debian/source/lintian-overrides', """\
blah source: patch-file-exists-but info
""")])
        self.assertEqual([
            Override(package='blah', archlist=None, type='source',
                     tag='patch-file-exists-but', info='info')],
            list(_get_overrides()))
        self.assertTrue(override_exists('patch-file-exists-but', info='info'))
        self.assertFalse(override_exists('patch-file-exists-but', info='no'))
        self.assertTrue(override_exists(
            'patch-file-exists-but', info='info', package='source'))



#!/usr/bin/python3

from lintian_brush.control import (
    guess_template_type,
    )
from lintian_brush.reformatting import (
    check_generated_file,
    GeneratedFile,
    )


def rewrite_line(line):
    if not line.split(b'#', 1)[0].strip():
        return line
    if line.startswith(b' ') or line.startswith(b'\t'):
        return line
    (key, value) = line.split(b':', 1)
    if not value.strip():
        return line
    return b'%s: %s\n' % (key, value.lstrip().rstrip(b'\n'))


def fix_field_spacing(path):
    lines = []
    changed = False
    with open(path, 'rb') as f:
        for oldline in f:
            newline = rewrite_line(oldline)
            if newline != oldline:
                changed = True
            lines.append(newline)
    if not changed:
        return False
    with open(path, 'wb') as f:
        f.writelines(lines)
    return True


try:
    check_generated_file('debian/control')
except GeneratedFile as e:
    if e.template_path:
        template_type = guess_template_type(e.template_path)
        if template_type is None:
            raise
        changed = fix_field_spacing(e.template_path)
        if changed:
            fix_field_spacing('debian/control')
    else:
        raise
else:
    changed = fix_field_spacing('debian/control')

if changed:
    print('Strip unusual field spacing from debian/control.')
    print('Fixed-Lintian-Tags: debian-control-has-unusual-field-spacing')



#!/usr/bin/python3

from lintian_brush.control import (
    add_dependency,
    parse_relations,
    ControlUpdater,
    )
from lintian_brush.fixer import report_result


uses_debhelper = False
misc_depends_added = []


with ControlUpdater() as updater:
    for entry in parse_relations(updater.source.get("Build-Depends", '')):
        (head_whitespace, relation, tail_whitespace) = entry
        if any(r.name == 'debhelper' for r in relation):
            uses_debhelper = True

    if uses_debhelper:
        for binary in updater.binaries:
            for entry in parse_relations(binary.get("Depends", '')):
                (head_whitespace, relation, tail_whitespace) = entry
                if any(r.name == '${misc:Depends}' for r in relation):
                    break
            else:
                binary["Depends"] = add_dependency(
                    binary.get("Depends", ''), "${misc:Depends}")
                misc_depends_added.append(binary["Package"])


report_result(
    "Add missing ${misc:Depends} to Depends for %s." %
    ", ".join(misc_depends_added),
    fixed_lintian_tags=['debhelper-but-no-misc-depends'])



#!/usr/bin/python3

from lintian_brush.systemd import systemd_service_files, SystemdServiceUpdater


for path in systemd_service_files():
    with SystemdServiceUpdater(path) as updater:
        val = updater.file['Service']['PIDFile']
        if isinstance(val, str):
            updater.file['Service']['PIDFile'] = val.replace(
                "/var/run/", "/run/")


print("Replace /var/run with /run for the Service PIDFile.")
print("Fixed-Lintian-Tags: systemd-service-file-pidfile-refers-to-var-run")



#!/usr/bin/python3

# Copyright (C) 2018 Jelmer Vernooij
#
# Parts imported from python3-iniparse:
# Copyright (c) 2001, 2002, 2003 Python Software Foundation
# Copyright (c) 2004-2008 Paramjit Oberoi <param.cs.wisc.edu>
# Copyright (c) 2007 Tim Lauridsen <tla@rasmil.dk>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Utility functions for dealing with systemd files."""

from io import StringIO

from iniparse.config import ConfigNamespace
from iniparse.ini import (
    LineContainer,
    SectionLine,
    OptionLine,
    ContinuationLine,
    MissingSectionHeaderError,
    ParsingError,
    EmptyLine,
    CommentLine,
    make_comment,
    readline_iterator,
    )

from .reformatting import Updater

import os

LIST_KEYS = [
    'Before', 'After', 'Documentation', 'Wants', 'Alias', 'WantedBy',
    'Requires', 'RequiredBy', 'Also', 'Conflicts']


class Section(ConfigNamespace):
    _lines = None
    _options = None

    def __init__(self, lineobj):
        self._lines = [lineobj]
        self._options = {}

    def rename(self, oldkey, newkey):
        option = self._options[oldkey]
        self._options[newkey] = option
        del self._options[oldkey]
        if isinstance(option, list):
            for opt in option:
                opt.name = newkey
        else:
            option.name = newkey

    def _getitem(self, key):
        if key == '__name__':
            return self._lines[-1].name
        try:
            option = self._options[key]
        except KeyError:
            if key in LIST_KEYS:
                return OptionList(self, key, [])
            raise
        if isinstance(option, list):
            return OptionList(self, key, option)
        else:
            return option.value

    def __setitem__(self, key, value):
        if key in LIST_KEYS:
            remaining = list(value)
            for option in self._options.get(key, []):
                values = option.value.split()
                for v in list(values):
                    try:
                        remaining.remove(v)
                    except ValueError:
                        values.remove(v)
                option.value = ' '.join(values)
                if not option.value:
                    for l in self._lines:
                        l.contents.remove(option)
            if remaining:
                obj = LineContainer(
                    OptionLine(key, ' '.join(remaining), separator='='))
                self._lines[-1].add(obj)
                self._options[key].append(obj)
        else:
            if key not in self._options:
                # create a dummy object - value may have multiple lines
                obj = LineContainer(OptionLine(key, '', separator='='))
                self._lines[-1].add(obj)
                self._options[key] = obj
            # the set_value() function in LineContainer
            # automatically handles multi-line values
            self._options[key].value = value

    def __delitem__(self, key):
        for l in self._lines:
            remaining = []
            for o in l.contents:
                if isinstance(o, LineContainer):
                    if key != o.name:
                        remaining.append(o)
                else:
                    remaining.append(o)
            l.contents = remaining
        del self._options[key]

    def __iter__(self):
        d = set()
        for l in self._lines:
            for x in l.contents:
                if isinstance(x, LineContainer):
                    ans = x.name
                    if ans not in d:
                        yield ans
                        d.add(ans)

    def _new_namespace(self, name):
        raise Exception('No sub-sections allowed', name)


class OptionList(object):

    def __init__(self, section, key, options):
        self._section = section
        self._key = key
        self._options = options

    def __getitem__(self, i):
        return self._items()[i].value

    def __delitem__(self, k):
        i = 0
        for o in self._options:
            vals = o.value.split()
            if k >= i and k < i + len(vals):
                del vals[k - i]
                o.value = ' '.join(vals)
                for l in self._section._lines:
                    l.contents.remove(o)
                break
            i += len(vals)
        else:
            raise IndexError(k)

    def __setitem__(self, k, val):
        i = 0
        for o in self._options:
            vals = o.value.split()
            if k >= i and k < i + len(vals):
                vals[k - i] = val
                o.value = ' '.join(vals)
                break
            i += len(vals)
        else:
            raise IndexError(k)

    def _items(self):
        ret = []
        for o in self._options:
            ret.extend(o.value.split())
        return ret

    def __iter__(self):
        return iter(self._items())

    def __len__(self):
        return len(self._items())

    def __contains__(self, v):
        return v in self._items()

    def remove(self, value):
        for i, v in enumerate(self._items()):
            if v == value:
                del self[i]
                return
        raise ValueError(value)

    def append(self, v):
        option = OptionLine(self._key, v, separator='=')
        obj = LineContainer(option)
        self._section._lines[-1].add(obj)
        self._section._options[v].append(obj)
        self._options.append(option)


class UnitFile(ConfigNamespace):
    _data = None
    _sections = None

    def __init__(self, fp=None):
        self._data = LineContainer()
        self._sections = {}
        if fp is not None:
            self._readfp(fp)

    def _getitem(self, key):
        return self._sections[key]

    def __setitem__(self, key, value):
        raise Exception('Values must be inside sections', key, value)

    def __delitem__(self, key):
        for line in self._sections[key]._lines:
            self._data.contents.remove(line)
        del self._sections[key]

    def __iter__(self):
        d = set()
        for x in self._data.contents:
            if isinstance(x, LineContainer):
                if x.name not in d:
                    yield x.name
                    d.add(x.name)

    def _new_namespace(self, name):
        if self._data.contents:
            self._data.add(EmptyLine())
        obj = LineContainer(SectionLine(name))
        self._data.add(obj)
        if name in self._sections:
            ns = self._sections[name]
            ns._lines.append(obj)
        else:
            ns = Section(obj)
            self._sections[name] = ns
        return ns

    def __str__(self):
        return self._data.__str__()

    _line_types = [EmptyLine, CommentLine,
                   SectionLine, OptionLine,
                   ContinuationLine]

    def _parse(self, line):
        for linetype in self._line_types:
            lineobj = linetype.parse(line)
            if lineobj:
                return lineobj
        else:
            # can't parse line
            return None

    def _readfp(self, fp):
        cur_section = None
        cur_option = None
        cur_section_name = None
        cur_option_name = None
        pending_lines = []
        try:
            fname = fp.name
        except AttributeError:
            fname = '<???>'
        linecount = 0
        exc = None
        line = None
        optobj = None

        for line in readline_iterator(fp):
            lineobj = self._parse(line)
            linecount += 1

            if not cur_section and not isinstance(
                    lineobj, (CommentLine, EmptyLine, SectionLine)):
                raise MissingSectionHeaderError(fname, linecount, line)

            if lineobj is None:
                if exc is None:
                    exc = ParsingError(fname)
                exc.append(linecount, line)
                lineobj = make_comment(line)

            if isinstance(lineobj, ContinuationLine):
                if cur_option:
                    if pending_lines:
                        cur_option.extend(pending_lines)
                        pending_lines = []
                    cur_option.add(lineobj)
                else:
                    # illegal continuation line - convert to comment
                    if exc is None:
                        exc = ParsingError(fname)
                    exc.append(linecount, line)
                    lineobj = make_comment(line)

            if isinstance(lineobj, OptionLine):
                if pending_lines:
                    cur_section.extend(pending_lines)
                    pending_lines = []
                cur_option = LineContainer(lineobj)
                cur_section.add(cur_option)
                cur_option_name = cur_option.name
                optobj = self._sections[cur_section_name]
                if cur_option_name in LIST_KEYS:
                    if not cur_option.value.split():
                        # Reset list.
                        optobj._options[cur_option_name] = []
                    else:
                        optobj._options.setdefault(cur_option_name, []).append(
                            cur_option)
                else:
                    optobj._options[cur_option_name] = cur_option

            if isinstance(lineobj, SectionLine):
                self._data.extend(pending_lines)
                pending_lines = []
                cur_section = LineContainer(lineobj)
                self._data.add(cur_section)
                cur_option = None
                cur_option_name = None
                cur_section_name = cur_section.name
                if cur_section_name not in self._sections:
                    self._sections[cur_section_name] = Section(cur_section)
                else:
                    self._sections[cur_section_name]._lines.append(
                        cur_section)

            if isinstance(lineobj, (CommentLine, EmptyLine)):
                pending_lines.append(lineobj)

        self._data.extend(pending_lines)
        if line and line[-1] == '\n':
            self._data.add(EmptyLine())

        if exc:
            raise exc


def systemd_service_files(path='debian', exclude_links=True):
    """List paths to systemd service files."""
    for name in os.listdir(path):
        if not name.endswith('.service'):
            continue
        subpath = os.path.join(path, name)
        if exclude_links and os.path.islink(subpath):
            continue
        yield subpath


class SystemdServiceUpdater(Updater):

    def _parse(self, content):
        return UnitFile(StringIO(self._orig_content))

    def _format(self, parsed):
        return str(parsed)

    @property
    def file(self):
        return self._parsed


def update_service_file(path, cb):
    """Update a service file.

    This is a fairly stupid ini-style parser; it attempts
    to preserve formatting as best as it can. Other
    ini-style parsers like configobj and configparser
    don't preserve whitespace or comments.

    Args:
      path: Path to the service file
      cb: Callback called with a config.ConfigObj object
    """
    with SystemdServiceUpdater(path) as updater:
        for section in updater.file:
            for name in updater.file[section]:
                oldval = updater.file[section][name]
                newval = cb(section, name, oldval)
                if newval != oldval:
                    updater.file[section][name] = newval


def update_service(cb):
    """Update all service files.

    Args:
      cb: Callback called with a config.ConfigObj object
    """
    for path in systemd_service_files():
        update_service_file(path, cb)



#!/usr/bin/python3

from lintian_brush.systemd import systemd_service_files, SystemdServiceUpdater

for path in systemd_service_files():
    with SystemdServiceUpdater(path) as updater:
        unit = updater.file['Unit']
        try:
            unit.rename('BindTo', 'BindsTo')
        except KeyError:
            pass


print('Rename BindTo key to BindsTo in systemd files.')
print('Fixed-Lintian-Tags: systemd-service-file-refers-to-obsolete-bindto')



#!/usr/bin/python
# Copyright (C) 2019 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Tests for lintian_brush.upstream_metadata."""

from breezy.tests import (
    TestCase,
    TestCaseWithTransport,
    )


from lintian_brush.upstream_metadata import (
    UpstreamDatum,
    guess_repo_from_url,
    guess_from_package_json,
    guess_from_debian_watch,
    guess_from_r_description,
    )


class GuessFromDebianWatchTests(TestCaseWithTransport):

    def test_empty(self):
        self.build_tree_contents([('watch', """\
# Blah
""")])
        self.assertEqual(
            [], list(guess_from_debian_watch('watch', False)))

    def test_simple(self):
        self.build_tree_contents([('watch', """\
version=4
https://github.com/jelmer/dulwich/tags/dulwich-(.*).tar.gz
""")])
        self.assertEqual(
            [UpstreamDatum(
                'Repository', 'https://github.com/jelmer/dulwich.git',
                'likely')],
            list(guess_from_debian_watch('watch', False)))


class GuessFromPackageJsonTests(TestCaseWithTransport):

    def test_simple(self):
        self.build_tree_contents([('package.json', """\
{
  "name": "autosize",
  "version": "4.0.2",
  "author": {
    "name": "Jack Moore",
    "url": "http://www.jacklmoore.com",
    "email": "hello@jacklmoore.com"
  },
  "main": "dist/autosize.js",
  "license": "MIT",
  "homepage": "http://www.jacklmoore.com/autosize",
  "demo": "http://www.jacklmoore.com/autosize",
  "repository": {
    "type": "git",
    "url": "http://github.com/jackmoore/autosize.git"
  }
}
""")])
        self.assertEqual(
            [UpstreamDatum('Name', 'autosize', 'certain'),
             UpstreamDatum(
                 'Homepage', 'http://www.jacklmoore.com/autosize', 'certain'),
             UpstreamDatum(
                 'Repository', 'https://github.com/jackmoore/autosize.git',
                 'certain')],
            list(guess_from_package_json('package.json', False)))

    def test_dummy(self):
        self.build_tree_contents([('package.json', """\
{
  "name": "mozillaeslintsetup",
  "description": "This package file is for setup of ESLint.",
  "repository": {},
  "license": "MPL-2.0",
  "dependencies": {
    "eslint": "4.18.1",
    "eslint-plugin-html": "4.0.2",
    "eslint-plugin-mozilla": "file:tools/lint/eslint/eslint-plugin-mozilla",
    "eslint-plugin-no-unsanitized": "2.0.2",
    "eslint-plugin-react": "7.1.0",
    "eslint-plugin-spidermonkey-js":
        "file:tools/lint/eslint/eslint-plugin-spidermonkey-js"
  },
  "devDependencies": {}
}
""")])
        self.assertEqual(
            [UpstreamDatum('Name', 'mozillaeslintsetup', 'certain')],
            list(guess_from_package_json('package.json', False)))


class GuessFromRDescriptionTests(TestCaseWithTransport):

    def test_read(self):
        self.build_tree_contents([('DESCRIPTION', """\
Package: crul
Title: HTTP Client
Description: A simple HTTP client, with tools for making HTTP requests,
    and mocking HTTP requests. The package is built on R6, and takes
    inspiration from Ruby's 'faraday' gem (<https://rubygems.org/gems/faraday>)
    The package name is a play on curl, the widely used command line tool
    for HTTP, and this package is built on top of the R package 'curl', an
    interface to 'libcurl' (<https://curl.haxx.se/libcurl>).
Version: 0.8.4
License: MIT + file LICENSE
Authors@R: c(
    person("Scott", "Chamberlain", role = c("aut", "cre"),
    email = "myrmecocystus@gmail.com",
    comment = c(ORCID = "0000-0003-1444-9135"))
    )
URL: https://github.com/ropensci/crul (devel)
        https://ropenscilabs.github.io/http-testing-book/ (user manual)
        https://www.example.com/crul (homepage)
BugReports: https://github.com/ropensci/crul/issues
Encoding: UTF-8
Language: en-US
Imports: curl (>= 3.3), R6 (>= 2.2.0), urltools (>= 1.6.0), httpcode
        (>= 0.2.0), jsonlite, mime
Suggests: testthat, fauxpas (>= 0.1.0), webmockr (>= 0.1.0), knitr
VignetteBuilder: knitr
RoxygenNote: 6.1.1
X-schema.org-applicationCategory: Web
X-schema.org-keywords: http, https, API, web-services, curl, download,
        libcurl, async, mocking, caching
X-schema.org-isPartOf: https://ropensci.org
NeedsCompilation: no
Packaged: 2019-08-02 19:58:21 UTC; sckott
Author: Scott Chamberlain [aut, cre] (<https://orcid.org/0000-0003-1444-9135>)
Maintainer: Scott Chamberlain <myrmecocystus@gmail.com>
Repository: CRAN
Date/Publication: 2019-08-02 20:30:02 UTC
""")])
        ret = guess_from_r_description('DESCRIPTION', True)
        self.assertEqual(list(ret), [
            UpstreamDatum('Name', 'crul', 'certain'),
            UpstreamDatum('Archive', 'CRAN', 'certain'),
            UpstreamDatum(
                'Bug-Database', 'https://github.com/ropensci/crul/issues',
                'certain'),
            UpstreamDatum(
                'Repository', 'https://github.com/ropensci/crul', 'certain'),
            UpstreamDatum(
                'Homepage', 'https://www.example.com/crul', 'certain')])


class GuessRepoFromUrlTests(TestCase):

    def test_github(self):
        self.assertEqual(
            'https://github.com/jelmer/blah',
            guess_repo_from_url('https://github.com/jelmer/blah'))
        self.assertEqual(
            'https://github.com/jelmer/blah',
            guess_repo_from_url('https://github.com/jelmer/blah/blob/README'))
        self.assertIs(
            None,
            guess_repo_from_url('https://github.com/jelmer'))

    def test_none(self):
        self.assertIs(None, guess_repo_from_url('https://www.jelmer.uk/'))

    def test_known(self):
        self.assertEqual(
            'http://code.launchpad.net/blah',
            guess_repo_from_url('http://code.launchpad.net/blah'))

    def test_launchpad(self):
        self.assertEqual(
            'https://code.launchpad.net/bzr',
            guess_repo_from_url('http://launchpad.net/bzr/+download'))

    def test_savannah(self):
        self.assertEqual(
            'https://git.savannah.gnu.org/git/auctex.git',
            guess_repo_from_url('https://git.savannah.gnu.org/git/auctex.git'))
        self.assertIs(
            None,
            guess_repo_from_url(
                'https://git.savannah.gnu.org/blah/auctex.git'))

    def test_bitbucket(self):
        self.assertEqual(
            'https://bitbucket.org/fenics-project/dolfin',
            guess_repo_from_url(
                'https://bitbucket.org/fenics-project/dolfin/downloads/'))



#!/usr/bin/python3

import sys
from lintian_brush.control import (
    ensure_minimum_version,
    get_relation,
    read_debian_compat_file,
    update_control,
    )


def bump_debhelper(control):
    try:
        get_relation(control.get("Build-Depends", ""), "debhelper-compat")
    except KeyError:
        pass
    else:
        return
    control["Build-Depends"] = ensure_minimum_version(
            control.get("Build-Depends", ""),
            "debhelper",
            "%s~" % minimum_version)

# Debian source package is not obliged to contain `debian/compat'.
# Firstly, it may not use debhelper; secondly it may use modern
# `debhelper-compat' dependency style.


try:
    minimum_version = read_debian_compat_file('debian/compat')
except FileNotFoundError:
    sys.exit(0)

update_control(source_package_cb=bump_debhelper)

print("Bump debhelper dependency to >= %s, since that's what is "
      "used in debian/compat." % minimum_version)
print("Fixed-Lintian-Tags: package-needs-versioned-debhelper-build-depends")



#!/usr/bin/python3
# Copyright (C) 2019 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Support for accessing the DebBugs database."""


class DebBugs(object):
    """Read DebBugs data through UDD."""

    def __init__(self):
        self._conn = None

    async def connect(self):
        from .udd import connect_udd_mirror
        self._conn = await connect_udd_mirror()

    async def check_bug(self, package, bugid):
        """Check that a bug belongs to a particular package.

        Args:
          package: Package name
          bugid: Bug number
        Returns:
          Boolean
        """
        assert self._conn is not None, "call connect() first"
        row = await self._conn.fetchrow("""
select package from bugs where id = $1""", bugid)
        if row is None:
            return False
        return row[0] == package
